SIGHASH_ALL = 1
SIGHASH_NONE = 2
SIGHASH_SINGLE = 3
SIGHASH_FORKID = 0x40
SIGHASH_ANYONECANPAY = 0x80

VERIFY_NONE = 0

# Evaluate P2SH subscripts (softfork safe, BIP16).
VERIFY_P2SH = (1 << 0)

# Passing a non-strict-DER signature or one with undefined hashtype to a checksig
# operation causes script failure. Evaluating a pubkey that is not (0x04 + 64 bytes)
# or (0x02 or 0x03 + 32 bytes) by checksig causes script failure.
# (softfork safe, but not used or intended as a consensus rule).
VERIFY_STRICTENC = (1 << 1)

# Passing a non-strict-DER signature to a checksig operation causes script failure
# (softfork safe, BIP62 rule 1)
VERIFY_DERSIG = (1 << 2)

# Passing a non-strict-DER signature or one with S > order/2 to a checksig operation
# causes script failure (softfork safe, BIP62 rule 5).
VERIFY_LOW_S = (1 << 3)

# verify dummy stack item consumed by CHECKMULTISIG is of zero-length (softfork safe,
# BIP62 rule 7).
VERIFY_NULLDUMMY = (1 << 4)

# Using a non-push operator in the scriptSig causes script failure (softfork safe,
# BIP62 rule 2).
VERIFY_SIGPUSHONLY = (1 << 5)

# Require minimal encodings for all push operations (OP_0... OP_16, OP_1NEGATE where
# possible, direct pushes up to 75 bytes, OP_PUSHDATA up to 255 bytes, OP_PUSHDATA2 for
# anything larger). Evaluating any other push causes the script to fail (BIP62 rule 3).
# In addition, whenever a stack element is interpreted as a number, it must be of minimal
# length (BIP62 rule 4). (softfork safe)
VERIFY_MINIMALDATA = (1 << 6)

# Discourage use of NOPs reserved for upgrades (NOP1-10)
# Provided so that nodes can avoid accepting or mining transactions
# containing executed NOP's whose meaning may change after a soft-fork,
# thus rendering the script invalid; with this flag set executing
# discouraged NOPs fails the script. This verification flag will never be
# a mandatory flag applied to scripts in a block. NOPs that are not
# executed, e.g.  within an unexecuted IF ENDIF block, are *not* rejected.
VERIFY_DISCOURAGE_UPGRADABLE_NOPS = (1 << 7)

VERIFY_CLEANSTACK = (1 << 8)

# Verify CHECKLOCKTIMEVERIFY
#
# See BIP65 for details.
VERIFY_CHECKLOCKTIMEVERIFY = (1 << 9)

# support CHECKSEQUENCEVERIFY opcode
# See BIP112 for details
VERIFY_CHECKSEQUENCEVERIFY = (1 << 10)

# Support segregated witness
VERIFY_WITNESS = (1 << 11)

# Making v1-v16 witness program non-standard
VERIFY_DISCOURAGE_UPGRADABLE_WITNESS_PROGRAM = (1 << 12)

# Segwit script only: Require the argument of OP_IF/NOTIF to be exactly 0x01 or empty vector
VERIFY_MINIMALIF = (1 << 13)

# Signature(s) must be empty vector if an CHECK(MULTI)SIG operation failed
VERIFY_NULLFAIL = (1 << 14)

# Public keys in segregated witness scripts must be compressed
VERIFY_WITNESS_PUBKEYTYPE = (1 << 15)

# If sequence encodes a relative lock-time and this flag
# is set, the relative lock-time has units of 512 seconds,
# otherwise it specifies blocks with a granularity of 1.
SEQUENCE_LOCKTIME_TYPE_FLAG = (1 << 22)

# If this flag set, sequence is NOT interpreted as a
# relative lock-time.
SEQUENCE_LOCKTIME_DISABLE_FLAG = (1 << 31)



from pycoin.coins.SolutionChecker import ScriptError

from . import errno


class IntStreamer(object):

    @classmethod
    def int_from_script_bytes(class_, s, require_minimal=False):
        if len(s) == 0:
            return 0
        s = bytearray(s)
        s.reverse()
        i = s[0]
        v = i & 0x7f
        if require_minimal:
            if v == 0:
                if len(s) <= 1 or ((s[1] & 0x80) == 0):
                    raise ScriptError("non-minimally encoded", errno.UNKNOWN_ERROR)
        is_negative = ((i & 0x80) > 0)
        for b in s[1:]:
            v <<= 8
            v += b
        if is_negative:
            v = -v
        return v

    @classmethod
    def int_to_script_bytes(class_, v):
        if v == 0:
            return b''
        is_negative = (v < 0)
        if is_negative:
            v = -v
        ba = bytearray()
        while v >= 256:
            ba.append(v & 0xff)
            v >>= 8
        ba.append(v & 0xff)
        if ba[-1] >= 128:
            ba.append(0x80 if is_negative else 0)
        elif is_negative:
            ba[-1] |= 0x80
        return bytes(ba)



#!/usr/bin/env python

from __future__ import print_function

import argparse
import json
import re
import subprocess
import sys

from pycoin.encoding.hexbytes import h2b
from pycoin.networks.default import get_current_netcode
from pycoin.networks.registry import network_codes, network_for_netcode


HASH160_RE = re.compile(r"^([0-9a-fA-F]{40})$")


def gpg_entropy():
    try:
        output = subprocess.Popen(
            ["gpg", "--gen-random", "2", "64"], stdout=subprocess.PIPE).communicate()[0]
        return output
    except OSError:
        sys.stderr.write("warning: can't open gpg, can't use as entropy source\n")
    return b''


def get_entropy():
    entropy = bytearray()
    try:
        entropy.extend(gpg_entropy())
    except Exception:
        print("warning: can't use gpg as entropy source", file=sys.stdout)
    try:
        entropy.extend(open("/dev/random", "rb").read(64))
    except Exception:
        print("warning: can't use /dev/random as entropy source", file=sys.stdout)
    entropy = bytes(entropy)
    if len(entropy) < 64:
        raise OSError("can't find sources of entropy")
    return entropy


def create_output(item, key, network, output_key_set, subkey_path=None):
    key._network = network
    output_dict = {}
    output_order = []

    def add_output(json_key, value=None, human_readable_key=None):
        if output_key_set and json_key not in output_key_set:
            return
        if human_readable_key is None:
            human_readable_key = json_key.replace("_", " ")
        if value:
            if human_readable_key == "legacy":
                output_dict[json_key.strip()] = value
            else:
                output_dict[json_key.strip().lower()] = value
                output_order.append((json_key.lower(), human_readable_key))

    full_network_name = "%s %s" % (network.network_name, network.subnet_name)
    add_output("input", item)
    add_output("network", full_network_name)
    add_output("symbol", network.symbol)

    if hasattr(key, "output"):
        for k, v, text in key.output():
            add_output(k, v, text)

    if hasattr(key, "hwif"):
        if subkey_path:
            add_output("subkey_path", subkey_path)
        for k, v, text in network.output_for_hwif(key.serialize(), network, subkey_path, add_output):
            add_output(k, v, text)

    secret_exponent = getattr(key, "secret_exponent", lambda: None)()
    if secret_exponent:
        for k, v, text in network.output_for_secret_exponent(secret_exponent):
            add_output(k, v, text)

    public_pair = getattr(key, "public_pair", lambda: None)()
    if public_pair:
        for k, v, text in network.output_for_public_pair(public_pair):
            add_output(k, v, text)

    return output_dict, output_order


def dump_output(output_dict, output_order):
    print('')
    max_length = max(len(v[1]) for v in output_order)
    for key, hr_key in output_order:
        space_padding = ' ' * (1 + max_length - len(hr_key))
        val = output_dict.get(key)
        if val is None:
            print(hr_key)
        else:
            if len(val) > 80:
                val = "%s\\\n%s%s" % (val[:66], ' ' * (5 + max_length), val[66:])
            print("%s%s: %s" % (hr_key, space_padding, val))


def create_parser():
    codes = network_codes()
    parser = argparse.ArgumentParser(
        description='Crypto coin utility ku ("key utility") to show'
        ' information about Bitcoin or other cryptocoin data structures.',
        epilog=('Known networks codes:\n  ' +
                ', '.join(['%s (%s)' % (i, network_for_netcode(i).full_name()) for i in codes]))
    )
    parser.add_argument('-w', "--wallet", help='show just Bitcoin wallet key', action='store_true')
    parser.add_argument('-W', "--wif", help='show just Bitcoin WIF', action='store_true')
    parser.add_argument('-a', "--address", help='show just Bitcoin address', action='store_true')
    parser.add_argument(
        '-u', "--uncompressed", help='show output in uncompressed form', action='store_true')
    parser.add_argument(
        '-P', "--public", help='only show public version of wallet keys', action='store_true')

    parser.add_argument('-j', "--json", help='output as JSON', action='store_true')

    parser.add_argument('-b', "--brief", nargs="*", help='brief output; display a single field')

    parser.add_argument('-s', "--subkey", help='subkey path (example: 0H/2/15-20)', default="")
    parser.add_argument('-n', "--network", help='specify network', choices=codes)
    parser.add_argument(
        "--override-network", help='override detected network type', default=None, choices=codes)

    parser.add_argument(
        'item', nargs="*", help='a BIP0032 wallet key string;'
        ' a WIF;'
        ' a bitcoin address;'
        ' an SEC (ie. a 66 hex chars starting with 02, 03 or a 130 hex chars starting with 04);'
        ' the literal string "create" to create a new wallet key using strong entropy sources;'
        ' P:wallet passphrase (NOT RECOMMENDED);'
        ' H:wallet passphrase in hex (NOT RECOMMENDED);'
        ' E:electrum value (either a master public, master private, or initial data);'
        ' secret_exponent (in decimal or hex);'
        ' x,y where x,y form a public pair (y is a number or one of the strings "even" or "odd");'
        ' hash160 (as 40 hex characters).'
        ' If this argument is missing, input data will be read from stdin.')
    return parser


def _create_bip32(network):
    max_retries = 64
    for _ in range(max_retries):
        try:
            return network.keys.bip32_seed(get_entropy())
        except ValueError as e:
            continue
    # Probably a bug if we get here
    raise RuntimeError("can't create BIP32 key")


def parse_key(item, networks):
    default_network = networks[0]
    if item == 'create':
        return None, _create_bip32(default_network)

    if HASH160_RE.match(item):
        # BRAIN DAMAGE: lame hack for now
        item = default_network.address.for_p2pkh(h2b(item))

    item = default_network.str(item)

    for network in networks:
        for f in "hierarchical_key private_key public_key address".split():
            v = getattr(network.parse, f)(item)
            if v:
                return network, v

    return None, None


def generate_output(args, output_dict, output_order):
    if args.json:
        # the python2 version of json.dumps puts an extra blank prior to the end of each line
        # the "replace" is a hack to make python2 produce the same output as python3
        print(json.dumps(output_dict, indent=3, sort_keys=True).replace(" \n", "\n"))
        return

    if len(output_order) == 0:
        print("no output: use -j option to see keys")
    elif len(output_dict) == 1:
        print(output_dict[output_order[0][0]])
    else:
        dump_output(output_dict, output_order)


def ku(args, parser):
    fallback_network = network_for_netcode(args.network or get_current_netcode())
    parse_networks = [fallback_network] + [network_for_netcode(netcode) for netcode in network_codes()]
    if args.network:
        parse_networks = [network_for_netcode(args.network)]

    override_network = None
    if args.override_network:
        # Override the network value, so we can take the same xpubkey and view what
        # the values would be on each other network type.
        override_network = network_for_netcode(args.override_network)

    def parse_stdin():
        return [item for item in sys.stdin.readline().strip().split(' ') if len(item) > 0]

    output_key_set = set(args.brief or [])
    if args.wallet:
        output_key_set.add("wallet_key")
    elif args.wif:
        output_key_set.add("wif_uncompressed" if args.uncompressed else "wif")
    elif args.address:
        output_key_set.add("address" + ("_uncompressed" if args.uncompressed else ""))

    items = args.item if len(args.item) > 0 else parse_stdin()

    for item in items:
        key_network, key = parse_key(item, parse_networks)
        if key is None:
            print("can't parse %s" % item, file=sys.stderr)
            continue

        display_network = override_network or key_network or fallback_network

        if hasattr(key, "subkeys"):
            key_iter = key.subkeys(args.subkey)
        else:
            key_iter = [key]
        for key in key_iter:
            if args.public:
                key = key.public_copy()

            output_dict, output_order = create_output(item, key, display_network, output_key_set)

            generate_output(args, output_dict, output_order)


def main():
    parser = create_parser()
    args = parser.parse_args()
    ku(args, parser)


if __name__ == '__main__':
    main()



import unittest

from pycoin.ecdsa.secp256k1 import secp256k1_generator


class SigningTest(unittest.TestCase):
    def test_sign(self):
        PART1 = ["47f7616ea6f9b923076625b4488115de1ef1187f760e65f89eb6f4f7ff04b012"]
        PART2 = [x * 64 for x in "123456789abcde"]
        VAL = 28832970699858290
        for se in PART1 + PART2:
            secret_exponent = int(se, 16)
            sig = secp256k1_generator.sign(secret_exponent, VAL)

            public_pair = secp256k1_generator * secret_exponent

            v = secp256k1_generator.verify(public_pair, VAL, sig)
            self.assertTrue(v)

            sig1 = (sig[0] + 1, sig[1])
            v = secp256k1_generator.verify(public_pair, VAL, sig1)
            self.assertFalse(v)

            public_pairs = secp256k1_generator.possible_public_pairs_for_signature(VAL, sig)
            self.assertIn(public_pair, public_pairs)


if __name__ == "__main__":
    unittest.main()



from pycoin.networks.bitcoinish import create_bitcoinish_network


network = create_bitcoinish_network(
    symbol="BTC", network_name="Bitcoin", subnet_name="mainnet",
    wif_prefix_hex="80", sec_prefix="BTCSEC:", address_prefix_hex="00", pay_to_script_prefix_hex="05",
    bip32_prv_prefix_hex="0488ade4", bip32_pub_prefix_hex="0488B21E", bech32_hrp="bc",
    magic_header_hex="F9BEB4D9", default_port=8333,
    dns_bootstrap=[
        "seed.bitcoin.sipa.be", "dnsseed.bitcoin.dashjr.org",
        "bitseed.xf2.org", "dnsseed.bluematt.me",
    ])



import unittest

from pycoin.crack.ecdsa import crack_secret_exponent_from_k, crack_k_from_sigs
from pycoin.ecdsa.secp256k1 import secp256k1_generator


def make_gen_k_const(K):

    def gen_k(*args):
        return K
    return gen_k


class CrackSigTest(unittest.TestCase):
    def test_crack_secret_exponent_from_k(self):
        k = 105
        se = 181919191
        gen_k = make_gen_k_const(k)
        val = 488819181819384
        sig = secp256k1_generator.sign(se, val, gen_k=gen_k)
        cracked_se = crack_secret_exponent_from_k(secp256k1_generator, val, sig, k)
        self.assertEqual(cracked_se, se)

    def test_crack_k_from_sigs(self):
        k = 105
        se = 181919191
        gen_k = make_gen_k_const(k)
        val1 = 488819181819384
        val2 = 588819181819384
        sig1 = secp256k1_generator.sign(se, val1, gen_k=gen_k)
        sig2 = secp256k1_generator.sign(se, val2, gen_k=gen_k)
        cracked_k = crack_k_from_sigs(secp256k1_generator, sig1, val1, sig2, val2)
        self.assertEqual(cracked_k, k)



#!/usr/bin/env python

from __future__ import print_function

import argparse
import calendar
import codecs
import datetime
import io
import os.path
import re
import sqlite3
import subprocess
import sys

from .dump import dump_tx

from pycoin.coins.exceptions import BadSpendableError
from pycoin.convention import tx_fee, satoshi_to_mbtc
from pycoin.encoding.hexbytes import b2h, h2b, h2b_rev
from pycoin.key.subpaths import subpaths_for_path_range
from pycoin.networks.registry import network_codes, network_for_netcode
from pycoin.networks.default import get_current_netcode
from pycoin.services import spendables_for_address, get_tx_db
from pycoin.services.providers import message_about_tx_cache_env, \
    message_about_tx_for_tx_hash_env, message_about_spendables_for_address_env


DEFAULT_VERSION = 1
DEFAULT_LOCK_TIME = 0


def range_int(min, max, name):

    def cast(v):
        v = int(v)
        if not (min <= v <= max):
            raise ValueError()
        return v

    cast.__name__ = name
    return cast


def validate_bitcoind(tx, tx_db, bitcoind_url):
    try:
        from pycoin.services.bitcoind import bitcoind_agrees_on_transaction_validity
        if bitcoind_agrees_on_transaction_validity(bitcoind_url, tx):
            print("interop test passed for %s" % tx.id(), file=sys.stderr)
        else:
            print("tx ==> %s FAILED interop test" % tx.id(), file=sys.stderr)
    except ImportError:
        print("warning: can't talk to bitcoind due to missing library")


def check_fees(tx):
    total_in, total_out = tx.total_in(), tx.total_out()
    actual_tx_fee = total_in - total_out
    recommended_tx_fee = tx_fee.recommended_fee_for_tx(tx)
    print("warning: transaction fees recommendations casually calculated and estimates may be incorrect",
          file=sys.stderr)
    if actual_tx_fee > recommended_tx_fee:
        print("warning: transaction fee of %s exceeds expected value of %s mBTC" %
              (satoshi_to_mbtc(actual_tx_fee), satoshi_to_mbtc(recommended_tx_fee)),
              file=sys.stderr)
    elif actual_tx_fee < 0:
        print("not enough source coins (%s mBTC) for destination (%s mBTC)."
              " Short %s mBTC" %
              (satoshi_to_mbtc(total_in),
               satoshi_to_mbtc(total_out), satoshi_to_mbtc(-actual_tx_fee)),
              file=sys.stderr)
    elif actual_tx_fee < recommended_tx_fee:
        print("warning: transaction fee lower than (casually calculated)"
              " expected value of %s mBTC, transaction might not propogate" %
              satoshi_to_mbtc(recommended_tx_fee), file=sys.stderr)
    return actual_tx_fee


EARLIEST_DATE = datetime.datetime(year=2009, month=1, day=1)


def parse_locktime(s):
    s = re.sub(r"[ ,:\-]+", r"-", s)
    for fmt1 in ["%Y-%m-%dT", "%Y-%m-%d", "%b-%d-%Y", "%b-%d-%y", "%B-%d-%Y", "%B-%d-%y"]:
        for fmt2 in ["T%H-%M-%S", "T%H-%M", "-%H-%M-%S", "-%H-%M", ""]:
            fmt = fmt1 + fmt2
            try:
                when = datetime.datetime.strptime(s, fmt)
                if when < EARLIEST_DATE:
                    raise ValueError("invalid date: must be after %s" % EARLIEST_DATE)
                return calendar.timegm(when.timetuple())
            except ValueError:
                pass
    return int(s)


parse_locktime.__name__ = 'locktime'


def parse_fee(fee):
    if fee in ["standard"]:
        return fee
    return int(fee)


def parse_script_index_hex(input):
    index_s, opcodes = input.split("/", 1)
    index = int(index_s)
    return (index, h2b(opcodes))


def create_parser():
    codes = network_codes()
    EPILOG = ('Files are binary by default unless they end with the suffix ".hex". ' +
              'Known networks codes:\n  ' +
              ', '.join(['%s (%s)' % (i, network_for_netcode(i).full_name()) for i in codes]))

    parser = argparse.ArgumentParser(
        description="Manipulate bitcoin (or alt coin) transactions.",
        epilog=EPILOG)

    parser.add_argument('-t', "--transaction-version", type=range_int(0, 255, "version"),
                        help='Transaction version, either 1 (default) or 3 (not yet supported).')

    parser.add_argument('-l', "--lock-time", type=parse_locktime, help='Lock time; either a block'
                        'index, or a date/time (example: "2014-01-01T15:00:00"')

    parser.add_argument('-n', "--network", default=get_current_netcode(), choices=codes,
                        help=('Default network code (environment variable PYCOIN_DEFAULT_NETCODE '
                              'or "BTC"=Bitcoin mainnet if unset'))

    parser.add_argument('-a', "--augment", action='store_true',
                        help='augment tx by adding any missing spendable metadata by fetching'
                             ' inputs from cache and/or web services')

    parser.add_argument('-s', "--verbose-signature", action='store_true',
                        help='Display technical signature details.')

    parser.add_argument("-i", "--fetch-spendables", metavar="address", action="append",
                        help='Add all unspent spendables for the given bitcoin address. This information'
                        ' is fetched from web services. With no outputs, incoming spendables will be printed.')

    parser.add_argument("-I", "--dump-inputs", action='store_true', help='Dump inputs to this transaction.')

    parser.add_argument(
        "-k", "--keychain", default=":memory:",
        help="path to keychain file for hierarchical key hints (SQLite3 file created with keychain tool)")

    parser.add_argument(
        "-K", "--key-paths", default="",
        help="Key path hints to search hiearachical private keys (example: 0/0H/0-20)")

    parser.add_argument('-f', "--private-key-file", metavar="path-to-private-keys", action="append", default=[],
                        help='file containing WIF or BIP0032 private keys. If file name ends with .gpg, '
                        '"gpg -d" will be invoked automatically. File is read one line at a time, and if '
                        'the file contains only one WIF per line, it will also be scanned for a bitcoin '
                        'address, and any addresses found will be assumed to be public keys for the given'
                        ' private key.',
                        type=argparse.FileType('r'))

    parser.add_argument('-g', "--gpg-argument", help='argument to pass to gpg (besides -d).', default='')

    parser.add_argument("--remove-tx-in", metavar="tx_in_index_to_delete", action="append", type=int,
                        help='remove a tx_in')

    parser.add_argument("--remove-tx-out", metavar="tx_out_index_to_delete", action="append", type=int,
                        help='remove a tx_out')

    parser.add_argument("--replace-input-script", metavar="tx_in_script_slash_hex", action="append", default=[],
                        type=parse_script_index_hex, help='replace an input script: arg looks like "1/796a"')

    parser.add_argument('-F', "--fee", help='fee, in satoshis, to pay on transaction, or '
                        '"standard" to auto-calculate. This is only useful if the "split pool" '
                        'is used; otherwise, the fee is automatically set to the unclaimed funds.',
                        default="standard", metavar="transaction-fee", type=parse_fee)

    parser.add_argument('-C', "--cache", help='force the resultant transaction into the transaction cache.'
                        ' Mostly for testing.', action='store_true'),

    parser.add_argument("--db", help='force the transaction expressed by the given hex '
                        'into a RAM-based transaction cache. Mostly for testing.', action="append"),

    parser.add_argument('-u', "--show-unspents", action='store_true',
                        help='show TxOut items for this transaction in Spendable form.')

    parser.add_argument('-b', "--bitcoind-url",
                        help='URL to bitcoind instance to validate against (http://user:pass@host:port).')

    parser.add_argument('-o', "--output-file", metavar="path-to-output-file", type=argparse.FileType('wb'),
                        help='file to write transaction to. This supresses most other output.')

    parser.add_argument('-d', "--disassemble", action='store_true',
                        help='Disassemble scripts.')

    parser.add_argument("--pdb", action="store_true", help='Enter PDB debugger on each script instruction.')

    parser.add_argument("--trace", action='store_true', help='Trace scripts.')

    parser.add_argument('-p', "--pay-to-script", metavar="pay-to-script", action="append",
                        help='a hex version of a script required for a pay-to-script'
                        'input (a bitcoin address that starts with 3)')

    parser.add_argument("--signature", metavar="known-good-signature", action="append",
                        help='a hex version of a signature that will be used if useful')

    parser.add_argument("--sec", metavar="known-sec", action="append",
                        help='a hex version of an SEC that will be used if useful')

    parser.add_argument('-P', "--pay-to-script-file", metavar="pay-to-script-file", nargs=1,
                        type=argparse.FileType('r'), help='a file containing hex scripts '
                        '(one per line) corresponding to pay-to-script inputs')

    parser.add_argument("--dump-signatures", action="store_true",
                        help="print signatures (for use with --signature)")

    parser.add_argument("--dump-secs", action="store_true",
                        help="print secs (for use with --sec)")

    parser.add_argument("--coinbase", type=str, help="add an input as a coinbase from the given address")

    parser.add_argument("argument", nargs="*", help='generic argument: can be a hex transaction id '
                        '(exactly 64 characters) to be fetched from cache or a web service;'
                        ' a transaction as a hex string; a path name to a transaction to be loaded;'
                        ' a spendable 4-tuple of the form tx_id/tx_out_idx/script_hex/satoshi_count '
                        'to be added to TxIn list; an address/satoshi_count to be added to the TxOut '
                        'list; an address or script to be added to the TxOut list and placed in the '
                        '"split pool".')

    return parser


def replace_with_gpg_pipe(args, f):
    gpg_args = ["gpg", "-d"]
    if args.gpg_argument:
        gpg_args.extend(args.gpg_argument.split())
    gpg_args.append(f.name)
    popen = subprocess.Popen(gpg_args, stdout=subprocess.PIPE)
    return popen.stdout


def parse_private_key_file(args, keychain, network):
    wif_re = re.compile(r"[1-9a-km-zA-LMNP-Z]{51,111}")
    # address_re = re.compile(r"[1-9a-kmnp-zA-KMNP-Z]{27-31}")
    for f in args.private_key_file:
        if f.name.endswith(".gpg"):
            f = replace_with_gpg_pipe(args, f)
        for line in f.readlines():
            # decode
            if isinstance(line, bytes):
                line = line.decode("utf8")
            # look for WIFs
            possible_keys = wif_re.findall(line)

            def make_key(x):
                try:
                    return network.parse.secret(x)
                except Exception:
                    return None

            keys = [make_key(x) for x in possible_keys]
            for key in keys:
                if key:
                    keychain.add_secrets([key])
                    keychain.add_key_paths(key, subpaths_for_path_range(args.key_paths))


TX_ID_RE = re.compile(r"^[0-9a-fA-F]{64}$")


def parse_tx(tx_class, arg, parser, tx_db, network):
    # hex transaction id
    tx = None

    if TX_ID_RE.match(arg):
        if tx_db is None:
            tx_db = create_tx_db(network)
        tx = tx_db.get(h2b_rev(arg))
        if not tx:
            parser.error("can't find Tx with id %s" % arg)
        return tx, tx_db

    # hex transaction data
    try:
        return tx_class.from_hex(arg), tx_db
    except Exception:
        pass

    if os.path.exists(arg):
        try:
            with open(arg, "rb") as f:
                if f.name.endswith("hex"):
                    f = io.BytesIO(codecs.getreader("hex_codec")(f).read())
                tx = tx_class.parse(f)
                tx.parse_unspents(f)
        except Exception:
            pass

    return tx, tx_db


def parse_scripts(args, keychain):
    warnings = []

    for p2s in args.pay_to_script or []:
        try:
            keychain.add_p2s_script(h2b(p2s))
        except Exception:
            warnings.append("warning: error parsing pay-to-script value %s" % p2s)

    hex_re = re.compile(r"[0-9a-fA-F]+")
    for f in args.pay_to_script_file or []:
        count = 0
        for l in f:
            try:
                m = hex_re.search(l)
                if m:
                    p2s = m.group(0)
                    keychain.add_p2s_script(h2b(p2s))
                    count += 1
            except Exception:
                warnings.append("warning: error parsing pay-to-script file %s" % f.name)
        if count == 0:
            warnings.append("warning: no scripts found in %s" % f.name)
    keychain.commit()
    return warnings


def create_tx_db(network):
    tx_db = get_tx_db(network.symbol)
    tx_db.warning_tx_cache = message_about_tx_cache_env()
    tx_db.warning_tx_for_tx_hash = message_about_tx_for_tx_hash_env(network.symbol)
    return tx_db


def parse_parts(tx_class, arg, spendables, payables, network):
    parts = arg.split("/")
    if 4 <= len(parts) <= 7:
        # spendable
        try:
            spendables.append(tx_class.Spendable.from_text(arg))
            return True
        except Exception:
            pass

    if len(parts) == 2:
        script = script_for_address_or_opcodes(network, parts[0])
        if script is not None:
            payables.append((script, parts[1]))
            return True


def key_found(arg, keychain, key_paths, network):
    try:
        secret = network.parse.secret(arg)
        if secret:
            # TODO: check network
            keychain.add_secrets([secret])
            keychain.add_key_paths(secret, subpaths_for_path_range(key_paths))
            return True
    except Exception:
        pass

    return False


def script_for_address_or_opcodes(network, text):
    try:
        script = network.contract.for_address(text)
        if script:
            return script
    except Exception:
        pass
    try:
        return network.script.compile(text)
    except Exception:
        pass


def build_coinbase_tx(network, address_or_opcodes):
    puzzle_script = script_for_address_or_opcodes(network, address_or_opcodes)
    txs_in = [network.tx.TxIn.coinbase_tx_in(b'fake-pycoin-coinbase')]
    txs_out = [network.tx.TxOut(int(50*1e8), puzzle_script)]
    tx = network.tx(1, txs_in, txs_out)
    return tx


def parse_context(args, parser):
    network = network_for_netcode(args.network)
    tx_class = network.tx

    # defaults

    spendables = []
    payables = []

    # we create the tx_db lazily
    tx_db = None

    if args.db:

        try:
            txs = [tx_class.from_hex(tx_hex) for tx_hex in args.db or []]
        except Exception:
            parser.error("can't parse ")

        the_ram_tx_db = dict((tx.hash(), tx) for tx in txs)
        if tx_db is None:
            tx_db = create_tx_db(network)
        tx_db.lookup_methods.append(the_ram_tx_db.get)

    txs = []

    if args.coinbase:
        coinbase_tx = build_coinbase_tx(network, args.coinbase)
        txs.append(coinbase_tx)

    keychain = network.keychain(sqlite3.connect(args.keychain))

    # there are a few warnings we might optionally print out, but only if
    # they are relevant. We don't want to print them out multiple times, so we
    # collect them here and print them at the end if they ever kick in.

    warning_spendables = None

    for arg in args.argument:
        tx, tx_db = parse_tx(tx_class, arg, parser, tx_db, network)
        if tx:
            txs.append(tx)
            continue

        if key_found(arg, keychain, args.key_paths, network):
            continue

        if parse_parts(tx_class, arg, spendables, payables, network):
            continue

        payable = script_for_address_or_opcodes(network, arg)
        if payable is not None:
            payables.append((payable, 0))
            continue

        parser.error("can't parse %s" % arg)

    parse_private_key_file(args, keychain, network)

    if args.fetch_spendables:
        warning_spendables = message_about_spendables_for_address_env(args.network)
        for address in args.fetch_spendables:
            spendables.extend(spendables_for_address(address, args.network))

    return (network, txs, spendables, payables, keychain, tx_db, warning_spendables)


def merge_txs(network, txs, spendables, payables):

    tx_class = network.tx
    txs_in = []
    txs_out = []
    unspents = []

    # we use a clever trick here to keep each tx_in corresponding with its tx_out
    for tx in txs:
        smaller = min(len(tx.txs_in), len(tx.txs_out))
        txs_in.extend(tx.txs_in[:smaller])
        txs_out.extend(tx.txs_out[:smaller])
        unspents.extend(tx.unspents[:smaller])
    for tx in txs:
        smaller = min(len(tx.txs_in), len(tx.txs_out))
        txs_in.extend(tx.txs_in[smaller:])
        txs_out.extend(tx.txs_out[smaller:])
        unspents.extend(tx.unspents[smaller:])
    for spendable in spendables:
        txs_in.append(spendable.tx_in())
        unspents.append(spendable)
    for script, coin_value in payables:
        txs_out.append(tx_class.TxOut(coin_value, script))

    return txs_in, txs_out, unspents


def calculate_lock_time_and_version(args, txs):

    # if no lock_time is explicitly set, inherit from the first tx or use default
    lock_time = args.lock_time
    if lock_time is None:
        if txs:
            lock_time = txs[0].lock_time
        else:
            lock_time = DEFAULT_LOCK_TIME

    # if no version is explicitly set, inherit from the first tx or use default
    version = args.transaction_version
    if version is None:
        if txs:
            version = txs[0].version
        else:
            version = DEFAULT_VERSION
    return lock_time, version


def remove_indices(items, indices):
    if indices:
        s = set(indices)
        items = [i for idx, i in enumerate(items) if idx not in s]
    return items


def replace_input_scripts(txs_in, replacements):
    for index, blob in replacements:
        txs_in[index].script = blob


def wif_iter(iters):
    while len(iters) > 0:
        for idx, iter in enumerate(iters):
            try:
                wif = next(iter)
                yield wif
            except StopIteration:
                iters = iters[:idx] + iters[idx+1:]
                break


def generate_tx(network, txs, spendables, payables, args):
    txs_in, txs_out, unspents = merge_txs(network, txs, spendables, payables)
    lock_time, version = calculate_lock_time_and_version(args, txs)
    if len(unspents) == len(txs_in):
        unspents = remove_indices(unspents, args.remove_tx_in)
    replace_input_scripts(txs_in, args.replace_input_script)
    txs_in = remove_indices(txs_in, args.remove_tx_in)
    txs_out = remove_indices(txs_out, args.remove_tx_out)
    tx = network.tx(txs_in=txs_in, txs_out=txs_out, lock_time=lock_time, version=version, unspents=unspents)
    fee = args.fee
    try:
        if len(payables) > 0:
            network.tx_utils.distribute_from_split_pool(tx, fee)
    except ValueError as ex:
        print("warning: %s" % ex.args[0], file=sys.stderr)
    return tx


def print_output(tx, include_unspents, output_file, show_unspents,
                 network, verbose_signature, disassembly_level, trace, pdb):
    if len(tx.txs_in) == 0:
        print("warning: transaction has no inputs", file=sys.stderr)

    if len(tx.txs_out) == 0:
        print("warning: transaction has no outputs", file=sys.stderr)

    tx_as_hex = tx.as_hex(include_unspents=include_unspents)

    if output_file:
        f = output_file
        if f.name.endswith(".hex"):
            f.write(tx_as_hex.encode("utf8"))
        else:
            tx.stream(f, include_unspents=include_unspents)
        f.close()
    elif show_unspents:
        for spendable in tx.tx_outs_as_spendable():
            print(spendable.as_text())
    elif len(tx.txs_out) == 0:
        for spendable in tx.unspents:
            print(spendable.as_text())
    else:
        if not tx.missing_unspents():
            check_fees(tx)
        output = []
        dump_tx(output, tx, network, verbose_signature, disassembly_level, trace, pdb)
        for line in output:
            print(line)
        if include_unspents:
            print("including unspents in hex dump since transaction not fully signed")
        print(tx_as_hex)


def do_signing(tx, keychain, p2sh_lookup, sec_hints, signature_hints, network):
    unsigned_before = tx.bad_solution_count()
    unsigned_after = unsigned_before
    if unsigned_before > 0 and (keychain.has_secrets() or sec_hints or signature_hints):
        print("signing...", file=sys.stderr)
        solver = tx.Solver(tx)
        solver.sign(keychain, p2sh_lookup=p2sh_lookup, sec_hints=sec_hints, signature_hints=signature_hints)

        unsigned_after = tx.bad_solution_count()
        if unsigned_after > 0:
            print("warning: %d TxIn items still unsigned" % unsigned_after, file=sys.stderr)
    return unsigned_after == 0


def cache_result(tx, tx_db, cache, network):
    if cache:
        if tx_db is None:
            tx_db = create_tx_db(network)
        tx_db.put(tx)
    return tx_db


def validate_tx(tx, tx_db, network):
    if not tx.txs_out:
        return
    if tx.missing_unspents():
        print("\n** can't validate transaction as source transactions missing", file=sys.stderr)
    else:
        try:
            if tx_db is None:
                tx_db = create_tx_db(network)
            tx.validate_unspents(tx_db)
            print('all incoming transaction values validated')
        except BadSpendableError as ex:
            print("\n**** ERROR: FEES INCORRECTLY STATED: %s" % ex.args[0], file=sys.stderr)
        except Exception as ex:
            print("\n*** can't validate source transactions as untampered: %s" %
                  ex.args[0], file=sys.stderr)


def validate_against_bitcoind(tx, tx_db, network, bitcoind_url):
    if bitcoind_url:
        if tx_db is None:
            tx_db = create_tx_db(network)
        validate_bitcoind(tx, tx_db, bitcoind_url)
    return tx_db


def dump_signatures_hex(tx, network):
    sigs = []
    for _, tx_in in enumerate(tx.txs_in):
        sigs.extend(network.who_signed.extract_signatures(tx, _))
    if len(sigs):
        print("SIGNATURES")
    for sig in sigs:
        print(b2h(sig[0]))


def dump_secs_hex(tx, network):
    sec_key_list = []
    for _, tx_in in enumerate(tx.txs_in):
        sec_key_list.extend(network.who_signed.extract_secs(tx, _))
    if len(sec_key_list):
        print("SECS")
    for sec in sec_key_list:
        print(b2h(sec))


def dump_inputs(tx, network):
    for _, tx_out in enumerate(tx.unspents):
        if tx_out:
            print("%d: %s %s" % (_, tx_out.coin_value, network.script.disassemble(tx_out.script)))
        else:
            print("%d: (missing spendable)" % _)


def tx(args, parser):
    (network, txs, spendables, payables, keychain, tx_db, warning_spendables) = parse_context(args, parser)

    for tx in txs:
        if tx.missing_unspents() and (args.augment or tx_db):
            if tx_db is None:
                tx_db = create_tx_db(network)
            tx.unspents_from_db(tx_db, ignore_missing=True)

    # build p2sh_lookup
    warnings = parse_scripts(args, keychain)
    for w in warnings:
        print(w)

    tx = generate_tx(network, txs, spendables, payables, args)

    signature_hints = [h2b(sig) for sig in (args.signature or [])]
    sec_hints = network.tx.solve.build_sec_lookup([h2b(sec) for sec in (args.sec or [])])

    is_fully_signed = do_signing(tx, keychain, keychain, sec_hints, signature_hints, network)

    include_unspents = not is_fully_signed

    if args.dump_signatures or args.dump_secs:
        if args.dump_signatures:
            dump_signatures_hex(tx, network)

        if args.dump_secs:
            dump_secs_hex(tx, network)

        return

    print_output(tx, include_unspents, args.output_file, args.show_unspents, network,
                 args.verbose_signature, args.disassemble, args.trace, args.pdb)

    tx_db = cache_result(tx, tx_db, args.cache, network)

    tx_db = validate_against_bitcoind(tx, tx_db, args.network, args.bitcoind_url)

    if args.dump_inputs:
        dump_inputs(tx, network)

    if not args.show_unspents:
        tx_db = validate_tx(tx, tx_db, network)

    # print warnings
    if tx_db:
        for m in [tx_db.warning_tx_cache, tx_db.warning_tx_for_tx_hash]:
            if m:
                print("warning: %s" % m, file=sys.stderr)
    if warning_spendables:
        print("warning: %s" % warning_spendables, file=sys.stderr)


def main():
    parser = create_parser()
    args = parser.parse_args()
    tx(args, parser)


if __name__ == '__main__':
    main()



import unittest

from pycoin.coins.bitcoin.SolutionChecker import TxContext
from pycoin.coins.bitcoin.VM import BitcoinVM
from pycoin.encoding.hexbytes import h2b
from pycoin.intbytes import int2byte
from pycoin.satoshi.opcodes import OPCODE_LIST
from pycoin.satoshi.IntStreamer import IntStreamer
from pycoin.symbols.btc import network


class ToolsTest(unittest.TestCase):

    def test_compile_push_data_list(self):

        def test_bytes(as_bytes):
            script = network.script.compile_push_data_list([as_bytes])
            # this is a pretty horrible hack to test the vm with long scripts. But it works
            tx_context = TxContext()
            tx_context.signature_for_hash_type_f = None
            tx_context.flags = 0
            tx_context.traceback_f = None
            vm = BitcoinVM(script, tx_context, tx_context.signature_for_hash_type_f, flags=0)
            vm.MAX_SCRIPT_LENGTH = int(1e9)
            vm.MAX_BLOB_LENGTH = int(1e9)
            stack = vm.eval_script()
            assert len(stack) == 1
            assert stack[0] == as_bytes

        def test_val(n):
            as_bytes = IntStreamer.int_to_script_bytes(n)
            test_bytes(as_bytes)

        for i in range(100):
            test_val(100)
        for i in range(0xfff0, 0x10004):
            test_val(i)
        for i in range(0xfffff0, 0x1000005):
            test_val(i)

        for l in (1, 2, 3, 254, 255, 256, 257, 258, 0xfff9, 0xfffe, 0xffff, 0x10000, 0x10001, 0x10005):
            for v in (1, 2, 3, 4, 15, 16, 17, 18):
                b = int2byte(v) * l
                test_bytes(b)

        b = int2byte(30) * (0x1000000+1)
        for l in (0x1000000-1, 0x1000000, 0x1000000+1):
            test_bytes(b[:l])

    def test_compile_decompile(self):
        def check(s):
            b1 = network.script.compile(s)
            s1 = network.script.disassemble(b1)
            b2 = network.script.compile(s1)
            self.assertEqual(s, s1)
            self.assertEqual(b1, b2)

        def build_hex(size, a, b):
            "build some random-looking hex"
            return "[%s]" % "".join("%02x" % (((i+a)*b) & 0xff) for i in range(size))

        check("[ff]")
        check("[ff03]")
        check("[ff030102]")
        check("[55aabbccddeeff112131]")
        long_hex_260 = build_hex(260, 13, 93)
        long_hex_270 = build_hex(270, 11, 47)
        check("%s %s" % (long_hex_260, long_hex_270))
        s = set(x[-1] for x in OPCODE_LIST)
        for opcode, code in OPCODE_LIST:
            # skip reassigned NOPs
            if opcode not in s:
                continue
            if opcode.startswith("OP_PUSHDATA"):
                # these disassemble differently
                continue
            check(opcode)

    def test_tx_7e0114e93f903892b4dff5526a8cab674b2825fd715c4a95f852a1aed634a0f6(self):
        # this tx is from testnet. We add an extra "OP_0" to the end
        # we need to check that the script is being disassembled correctly
        script = h2b(
            "0047304402201f994ca49451bc764fd090f31adb2fa4381b91f967dc05a6f538d4d1ba"
            "aa83cd02206ef3ad06de7890bc4130b4f57401412ca94897ea19b646f794a447237535"
            "1c1f0147304402201f994ca49451bc764fd090f31adb2fa4381b91f967dc05a6f538d4"
            "d1baaa83cd02204655e9eccac412407dfc3e5753a0f2ac605e41c7eb91630dc67137f2"
            "d8081c3a014d0b0152410479be667ef9dcbbac55a06295ce870b07029bfcdb2dce28d9"
            "59f2815b16f81798483ada7726a3c4655da4fbfc0e1108a8fd17b448a68554199c47d0"
            "8ffb10d4b84104c6047f9441ed7d6d3045406e95c07cd85c778e4b8cef3ca7abac09b9"
            "5c709ee51ae168fea63dc339a3c58419466ceaeef7f632653266d0e1236431a950cfe5"
            "2a4104f9308a019258c31049344f85f89d5229b531c845836f99b08601f113bce036f9"
            "388f7b0f632de8140fe337e62a37f3566500a99934c2231b6cb9fd7584b8e6724104e4"
            "93dbf1c10d80f3581e4904930b1404cc6c13900ee0758474fa94abe8c4cd1351ed993e"
            "a0d455b75642e2098ea51448d967ae33bfbdfe40cfe97bdc4773992254ae00")

        d1 = network.script.disassemble(script).split()
        self.assertEqual(len(d1), 5)
        self.assertEqual(d1[-1], "OP_0")

    def test_int_to_from_script_bytes(self):
        for i in range(-127, 127):
            self.assertEqual(IntStreamer.int_from_script_bytes(IntStreamer.int_to_script_bytes(i)), i)
        for i in range(-1024, 1024, 16):
            self.assertEqual(IntStreamer.int_from_script_bytes(IntStreamer.int_to_script_bytes(i)), i)
        for i in range(-1024*1024, 1024*1024, 10000):
            self.assertEqual(IntStreamer.int_from_script_bytes(IntStreamer.int_to_script_bytes(i)), i)
        self.assertEqual(IntStreamer.int_to_script_bytes(1), b"\1")
        self.assertEqual(IntStreamer.int_to_script_bytes(127), b"\x7f")
        self.assertEqual(IntStreamer.int_to_script_bytes(128), b"\x80\x00")

    def test_pr_364(self):
        """ See https://github.com/richardkiss/pycoin/pull/364
        """
        assert network.script.compile_push_data_list([None]) == b''


if __name__ == "__main__":
    unittest.main()



import io
import warnings

from ..Tx import Tx as BaseTx

from .ScriptTools import BitcoinScriptTools
from .Solver import BitcoinSolver as Solver
from .SolutionChecker import BitcoinSolutionChecker as SolutionChecker

from pycoin.convention import SATOSHI_PER_COIN
from pycoin.encoding.hash import double_sha256
from pycoin.encoding.hexbytes import b2h, b2h_rev, h2b_rev
from pycoin.satoshi.satoshi_struct import parse_struct, stream_struct
from pycoin.satoshi.satoshi_int import parse_satoshi_int
from pycoin.satoshi.satoshi_string import parse_satoshi_string, stream_satoshi_string

from ..exceptions import BadSpendableError, ValidationFailureError
from .TxIn import TxIn
from .TxOut import TxOut
from .Spendable import Spendable


MAX_MONEY = 21000000 * SATOSHI_PER_COIN
MAX_BLOCK_SIZE = 1000000

ZERO32 = b'\0' * 32


class Tx(BaseTx):
    TxIn = TxIn
    TxOut = TxOut
    Spendable = Spendable
    Solver = Solver
    SolutionChecker = SolutionChecker

    MAX_MONEY = MAX_MONEY
    MAX_TX_SIZE = MAX_BLOCK_SIZE

    ALLOW_SEGWIT = True

    @classmethod
    def coinbase_tx(cls, public_key_sec, coin_value, coinbase_bytes=b'', version=1, lock_time=0):
        """Create the special "first in block" transaction that includes the mining fees."""
        tx_in = cls.TxIn.coinbase_tx_in(script=coinbase_bytes)
        COINBASE_SCRIPT_OUT = "%s OP_CHECKSIG"
        script_text = COINBASE_SCRIPT_OUT % b2h(public_key_sec)
        script_bin = BitcoinScriptTools.compile(script_text)
        tx_out = cls.TxOut(coin_value, script_bin)
        return cls(version, [tx_in], [tx_out], lock_time)

    @classmethod
    def parse(class_, f, allow_segwit=None):
        """Parse a Bitcoin transaction Tx.

        :param f: a file-like object that contains a binary streamed transaction
        :param allow_segwit: (optional) set to True to allow parsing of segwit transactions.
            The default value is defined by the class variable ALLOW_SEGWIT
        """
        if allow_segwit is None:
            allow_segwit = class_.ALLOW_SEGWIT
        txs_in = []
        txs_out = []
        version, = parse_struct("L", f)
        v1 = ord(f.read(1))
        is_segwit = allow_segwit and (v1 == 0)
        v2 = None
        if is_segwit:
            flag = f.read(1)
            if flag == b'\0':
                raise ValueError("bad flag in segwit")
            if flag == b'\1':
                v1 = None
            else:
                is_segwit = False
                v2 = ord(flag)
        count = parse_satoshi_int(f, v=v1)
        txs_in = []
        for i in range(count):
            txs_in.append(class_.TxIn.parse(f))
        count = parse_satoshi_int(f, v=v2)
        txs_out = []
        for i in range(count):
            txs_out.append(class_.TxOut.parse(f))

        if is_segwit:
            for tx_in in txs_in:
                stack = []
                count = parse_satoshi_int(f)
                for i in range(count):
                    stack.append(parse_satoshi_string(f))
                tx_in.witness = stack
        lock_time, = parse_struct("L", f)
        return class_(version, txs_in, txs_out, lock_time)

    @classmethod
    def tx_from_hex(cls, hex_string):
        warnings.simplefilter('always', DeprecationWarning)
        warnings.warn("Call to deprecated function tx_from_hex, use from_hex instead",
                      category=DeprecationWarning, stacklevel=2)
        warnings.simplefilter('default', DeprecationWarning)
        return cls.from_hex(hex_string)

    def __init__(self, version, txs_in, txs_out, lock_time=0, unspents=None):
        """Tx constructor.

        :param version: version number of the Tx, usually 1
        :param txs_in: a list of :class:`Tx.TxIn <TxIn>` instances, which
            act as inputs to the transaction
        :param txs_out: a list of :class:`Tx.TxOut <TxOut>` instances, which
            act as outputs to the transaction
        :param lock_time: (optional) the lock time for the transaction, usually 0
        :param unspents: (optional) a list of :class:`Tx.Spendable <Spendable>` instances,
            which correspond to the entries referred to by txs_in
        :return: :class:`Tx`
        """
        self.version = version
        self.txs_in = txs_in
        self.txs_out = txs_out
        self.lock_time = lock_time
        self.unspents = unspents or []
        for tx_in in self.txs_in:
            assert type(tx_in) == self.TxIn
        for tx_out in self.txs_out:
            assert type(tx_out) == self.TxOut

    def stream(self, f, blank_solutions=False, include_unspents=False, include_witness_data=True):
        """Stream a Bitcoin transaction Tx to the file-like object f.

        :param f: writable file-like object to stream binary data of transaction
        :param blank_solutions: (optional) clear out the solutions scripts, effectively "unsigning" the
            transaction before writing it. Defaults to False
        :param include_unspents: (optional) stread out the Spendable objects after streaming the transaction.
            This is a pycoin-specific extension. Defaults to False.
        :param include_witness_data: (optional) stream segwit transactions including the witness data if the
            transaction has any witness data. Defaults to True.
        """
        include_witnesses = include_witness_data and self.has_witness_data()
        stream_struct("L", f, self.version)
        if include_witnesses:
            f.write(b'\0\1')
        stream_struct("I", f, len(self.txs_in))
        for t in self.txs_in:
            t.stream(f, blank_solutions=blank_solutions)
        stream_struct("I", f, len(self.txs_out))
        for t in self.txs_out:
            t.stream(f)
        if include_witnesses:
            for tx_in in self.txs_in:
                witness = tx_in.witness
                stream_struct("I", f, len(witness))
                for w in witness:
                    stream_satoshi_string(f, w)
        stream_struct("L", f, self.lock_time)
        if include_unspents and not self.missing_unspents():
            self.stream_unspents(f)

    def set_witness(self, tx_idx_in, witness):
        """Set the witness data for a given :class:`TxIn`.

        :param tx_idx_in: an integer index corresponding to the txs_in puzzle script entry that this is a witness to
        :param witness: a list of binary blobs that witness the solution to the given puzzle script
        """
        self.txs_in[tx_idx_in].witness = tuple(witness)

    def has_witness_data(self):
        """Return a boolean indicating if the transaction has any segwit data."""
        return any(len(tx_in.witness) > 0 for tx_in in self.txs_in)

    def hash(self, hash_type=None):
        """Return the binary hash for this :class:`Tx` object.

        :param hash_type: (optional) if set, generates a hash specific to a particular type of signature.

        :return: 32 byte long binary blob corresponding to the hash
        """
        s = io.BytesIO()
        self.stream(s, include_witness_data=False)
        if hash_type is not None:
            stream_struct("L", s, hash_type)
        return double_sha256(s.getvalue())

    def w_hash(self):
        """Return the segwit-specific binary hash for this :class:`Tx` object.

        :return: 32 byte long binary blob corresponding to the hash
        """
        return double_sha256(self.as_bin())

    def w_id(self):
        """Return the segwit-specific binary hash for this :class:`Tx` object as a hex string.
        Note that this is a ``reversed`` version of :func:`Tx.w_hash <w_hash>`.

        :return: 64 character long hex string corresponding to the hash
        """
        return b2h_rev(self.w_hash())

    def blanked_hash(self):
        """
        Return the hash for this Tx object with solution scripts blanked.
        This hash is useful for determining if two Txs might be equivalent modulo
        malleability. (That is, even if tx1 is morphed into tx2 using the malleability
        weakness, they will still have the same blanked hash.)

        :return: 32 byte long binary blob corresponding to the blanked hash
        """
        s = io.BytesIO()
        self.stream(s, blank_solutions=True)
        return double_sha256(s.getvalue())

    def total_out(self):
        return sum(tx_out.coin_value for tx_out in self.txs_out)

    def tx_outs_as_spendable(self, block_index_available=0):
        h = self.hash()
        return [
            self.Spendable.from_tx_out(tx_out, h, tx_out_index, block_index_available)
            for tx_out_index, tx_out in enumerate(self.txs_out)]

    def is_coinbase(self):
        return len(self.txs_in) == 1 and self.txs_in[0].is_coinbase()

    def __str__(self):
        return "Tx [%s]" % self.id()

    def __repr__(self):
        return "Tx [%s] (v:%d) [%s] [%s]" % (
            self.id(), self.version, ", ".join(str(t) for t in self.txs_in),
            ", ".join(str(t) for t in self.txs_out))

    def _check_tx_inout_count(self):
        if not self.txs_out:
            raise ValidationFailureError("txs_out = []")
        if not self.is_coinbase() and not self.txs_in:
            raise ValidationFailureError("txs_in = []")

    def _check_size_limit(self):
        size = len(self.as_bin())
        if size > self.MAX_TX_SIZE:
            raise ValidationFailureError("size > MAX_TX_SIZE")

    def _check_txs_out(self):
        # Check for negative or overflow output values
        nValueOut = 0
        for tx_out in self.txs_out:
            if tx_out.coin_value < 0 or tx_out.coin_value > self.MAX_MONEY:
                raise ValidationFailureError("tx_out value negative or out of range")
            nValueOut += tx_out.coin_value
            if nValueOut > self.MAX_MONEY:
                raise ValidationFailureError("tx_out total out of range")

    def _check_txs_in(self):
        # Check for duplicate inputs
        if [x for x in self.txs_in if self.txs_in.count(x) > 1]:
            raise ValidationFailureError("duplicate inputs")
        if (self.is_coinbase()):
            if not (2 <= len(self.txs_in[0].script) <= 100):
                raise ValidationFailureError("bad coinbase script size")
        else:
            refs = set()
            for tx_in in self.txs_in:
                if tx_in.previous_hash == ZERO32:
                    raise ValidationFailureError("prevout is null")
                pair = (tx_in.previous_hash, tx_in.previous_index)
                if pair in refs:
                    raise ValidationFailureError("spendable reused")
                refs.add(pair)

    def check(self):
        """
        Basic checks that don't depend on any context.
        Adapted from Bicoin Code: main.cpp
        """
        self._check_tx_inout_count()
        self._check_txs_out()
        self._check_txs_in()
        # Size limits
        self._check_size_limit()

    def bad_solution_count(self, *args, **kwargs):
        if self.is_coinbase():
            return 0
        return super(Tx, self).bad_solution_count(*args, **kwargs)

    """
    The functions below here deal with an optional additional parameter: "unspents".
    This parameter is a list of tx_out objects that are referenced by the
    list of self.tx_in objects.
    """

    def unspents_from_db(self, tx_db, ignore_missing=False):
        unspents = []
        for tx_in in self.txs_in:
            if tx_in.is_coinbase():
                unspents.append(None)
                continue
            tx = tx_db.get(tx_in.previous_hash)
            if tx and tx.hash() == tx_in.previous_hash:
                unspents.append(tx.txs_out[tx_in.previous_index])
            elif ignore_missing:
                unspents.append(None)
            else:
                raise KeyError(
                    "can't find tx_out for %s:%d" % (b2h_rev(tx_in.previous_hash), tx_in.previous_index))
        self.unspents = unspents

    def missing_unspent(self, idx):
        if self.is_coinbase():
            return True
        if len(self.unspents) <= idx:
            return True
        return self.unspents[idx] is None

    def missing_unspents(self):
        if self.is_coinbase():
            return False
        return (len(self.unspents) != len(self.txs_in) or
                any(self.missing_unspent(idx) for idx, tx_in in enumerate(self.txs_in)))

    def check_unspents(self):
        if self.missing_unspents():
            raise ValueError("wrong number of unspents. Call unspents_from_db or set_unspents.")

    def stream_unspents(self, f):
        self.check_unspents()
        for tx_out in self.unspents:
            if tx_out is None:
                tx_out = self.TxOut(0, b'')
            tx_out.stream(f)

    def parse_unspents(self, f):
        unspents = []
        for i in enumerate(self.txs_in):
            tx_out = self.TxOut.parse(f)
            if tx_out.coin_value == 0:
                tx_out = None
            unspents.append(tx_out)
        self.set_unspents(unspents)

    def total_in(self):
        if self.is_coinbase():
            return self.txs_out[0].coin_value
        self.check_unspents()
        return sum(tx_out.coin_value for tx_out in self.unspents)

    def fee(self):
        return self.total_in() - self.total_out()

    def validate_unspents(self, tx_db):
        """
        Spendable objects returned from blockchain.info or
        similar services contain coin_value information that must be trusted
        on faith. Mistaken coin_value data can result in coins being wasted
        to fees.

        This function solves this problem by iterating over the incoming
        transactions, fetching them from the tx_db in full, and verifying
        that the coin_values are as expected.

        Returns the fee for this transaction. If any of the spendables set by
        tx.set_unspents do not match the authenticated transactions, a
        ValidationFailureError is raised.
        """
        tx_hashes = set((tx_in.previous_hash for tx_in in self.txs_in))

        # build a local copy of the DB
        tx_lookup = {}
        for h in tx_hashes:
            if h == ZERO32:
                continue
            the_tx = tx_db.get(h)
            if the_tx is None:
                raise KeyError("hash id %s not in tx_db" % b2h_rev(h))
            if the_tx.hash() != h:
                raise KeyError("attempt to load Tx %s yielded a Tx with id %s" % (h2b_rev(h), the_tx.id()))
            tx_lookup[h] = the_tx

        for idx, tx_in in enumerate(self.txs_in):
            if tx_in.previous_hash == ZERO32:
                continue
            txs_out = tx_lookup[tx_in.previous_hash].txs_out
            if tx_in.previous_index > len(txs_out):
                raise BadSpendableError("tx_out index %d is too big for Tx %s" %
                                        (tx_in.previous_index, b2h_rev(tx_in.previous_hash)))
            tx_out1 = txs_out[tx_in.previous_index]
            tx_out2 = self.unspents[idx]
            if tx_out1.coin_value != tx_out2.coin_value:
                raise BadSpendableError(
                    "unspents[%d] coin value mismatch (%d vs %d)" % (
                        idx, tx_out1.coin_value, tx_out2.coin_value))
            if tx_out1.script != tx_out2.script:
                raise BadSpendableError("unspents[%d] script mismatch!" % idx)

        return self.fee()


"""
The MIT License (MIT)

Copyright (c) 2013 by Richard Kiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""



# generic solver

import pdb

from pycoin.coins.SolutionChecker import ScriptError
from pycoin.satoshi.checksigops import parse_signature_blob
from pycoin.encoding.exceptions import EncodingError
from pycoin.encoding.hash import hash160
from pycoin.encoding.hexbytes import b2h
from pycoin.encoding.sec import public_pair_to_sec, sec_to_public_pair
from pycoin.intbytes import indexbytes, int2byte
from pycoin.satoshi import der

from .constraints import Atom
from .ConstraintSolver import CONSTANT, VAR, LIST, SolvingError


DEFAULT_PLACEHOLDER_SIGNATURE = b''
DEFAULT_SIGNATURE_TYPE = 1


def _find_signatures(script_blobs, generator_for_signature_type_f, signature_for_hash_type_f, max_sigs, sec_keys):
    signatures = []
    secs_solved = set()
    seen = 0
    for data in script_blobs:
        if seen >= max_sigs:
            break
        try:
            sig_pair, signature_type = parse_signature_blob(data)
            generator = generator_for_signature_type_f(signature_type)
            seen += 1
            for idx, sec_key in enumerate(sec_keys):
                public_pair = sec_to_public_pair(sec_key, generator)
                sign_value = signature_for_hash_type_f(signature_type)
                v = generator.verify(public_pair, sign_value, sig_pair)
                if v:
                    signatures.append((idx, data))
                    secs_solved.add(sec_key)
                    break
        except (ValueError, EncodingError, der.UnexpectedDER, ScriptError):
            # if public_pair is invalid, we just ignore it
            pass
    return signatures, secs_solved


def hash_lookup_solver(m):

    def f(solved_values, **kwargs):
        the_hash = m["the_hash"]
        db = kwargs.get("hash160_lookup", {})
        result = db.get(the_hash)
        if result is None:
            result = kwargs.get("sec_hints", {}).get(the_hash)
            if result:
                return {m["1"]: result}
        if result is None:
            raise SolvingError("can't find public pair for %s" % b2h(the_hash))
        sec = public_pair_to_sec(result[1], compressed=result[2])
        return {m["1"]: sec}

    return (f, [m["1"]], ())


hash_lookup_solver.pattern = ('EQUAL', CONSTANT("the_hash"), ('HASH160', VAR("1")))


def constant_equality_solver(m):

    def f(solved_values, **kwargs):
        return {m["var"]: m["const"]}

    return (f, [m["var"]], ())


constant_equality_solver.pattern = ('EQUAL', VAR("var"), CONSTANT('const'))


def all_signature_hints(public_pair, signature_for_hash_type_f, **kwargs):
    default_sig_type = [kwargs.get("signature_type", DEFAULT_SIGNATURE_TYPE)]
    sig_hash_types_to_try = kwargs.get("sig_hash_types_to_try", default_sig_type)
    shfsh = kwargs.get("signature_hints_for_sig_hash")
    if shfsh:
        for sig_hash_type in sig_hash_types_to_try:
            sig_hash = signature_for_hash_type_f(sig_hash_type)
            for _ in shfsh.get(sig_hash, []):
                yield _
    shfpp = kwargs.get("signature_hints_for_public_pair")
    if shfpp:
        for _ in shfpp.get(public_pair, []):
            yield _
    for _ in kwargs.get("signature_hints", []):
        yield _


def signing_solver(m):
    def f(solved_values, **kwargs):
        signature_type = kwargs.get("signature_type", DEFAULT_SIGNATURE_TYPE)
        generator_for_signature_type_f = kwargs["generator_for_signature_type_f"]
        signature_for_hash_type_f = m["signature_for_hash_type_f"]
        existing_script = kwargs.get("existing_script", b'')
        existing_signatures, secs_solved = _find_signatures(
            existing_script, generator_for_signature_type_f, signature_for_hash_type_f,
            len(m["sig_list"]), m["sec_list"])

        sec_keys = m["sec_list"]
        signature_variables = m["sig_list"]

        signature_placeholder = kwargs.get("signature_placeholder", DEFAULT_PLACEHOLDER_SIGNATURE)

        db = kwargs.get("hash160_lookup", {})
        # we reverse this enumeration to make the behaviour look like the old signer. BRAIN DAMAGE
        for signature_order, sec_key in reversed(list(enumerate(sec_keys))):
            sec_key = solved_values.get(sec_key, sec_key)
            if sec_key in secs_solved:
                continue
            if len(existing_signatures) >= len(signature_variables):
                break
            result = db.get(hash160(sec_key))
            if result:
                secret_exponent = result[0]
                sig_hash = signature_for_hash_type_f(signature_type)
                generator = result[3]
                r, s = generator.sign(secret_exponent, sig_hash)
            else:
                # try existing signatures
                generator = generator_for_signature_type_f(signature_type)
                public_pair = sec_to_public_pair(sec_key, generator=generator)
                for sig in all_signature_hints(public_pair, signature_for_hash_type_f, **kwargs):
                    sig_hash = signature_for_hash_type_f(indexbytes(sig, -1))
                    sig_pair = der.sigdecode_der(sig[:-1])
                    if generator.verify(public_pair, sig_hash, sig_pair):
                        r, s = sig_pair
                        break
                else:
                    continue
            order = generator.order()
            if s + s > order:
                s = order - s
            binary_signature = der.sigencode_der(r, s) + int2byte(signature_type)
            existing_signatures.append((signature_order, binary_signature))

        # pad with placeholder signatures
        if signature_placeholder is not None:
            while len(existing_signatures) < len(signature_variables):
                existing_signatures.append((-1, signature_placeholder))
        existing_signatures.sort()
        return dict(zip(signature_variables, (es[-1] for es in existing_signatures)))
    return (f, m["sig_list"], [a for a in m["sec_list"] if isinstance(a, Atom)])


signing_solver.pattern = ('SIGNATURES_CORRECT', LIST("sec_list"), LIST("sig_list"),
                          CONSTANT("signature_for_hash_type_f"))


def register_all(solver):
    for t in [hash_lookup_solver, constant_equality_solver, signing_solver]:
        solver.register_solver(t.pattern, t)



import json
import unittest
import os

from pycoin.encoding.hexbytes import h2b
from pycoin.symbols.btc import network


errno = network.validator.errno
flags = network.validator.flags
ScriptError = network.validator.ScriptError

SCRIPT_TESTS_JSON = os.path.dirname(__file__) + '/data/script_tests.json'


class TestTx(unittest.TestCase):
    pass


def parse_flags(flag_string):
    v = 0
    if len(flag_string) > 0:
        for f in flag_string.split(","):
            v |= getattr(flags, "VERIFY_%s" % f)
    return v


def build_credit_tx(script_out_bin, coin_value=0):
    txs_in = [network.tx.TxIn(b'\0'*32, 4294967295, b'\0\0', sequence=4294967295)]
    txs_out = [network.tx.TxOut(coin_value, script_out_bin)]
    return network.tx(1, txs_in, txs_out)


def build_spending_tx(script_in_bin, credit_tx):
    txs_in = [network.tx.TxIn(credit_tx.hash(), 0, script_in_bin, sequence=4294967295)]
    txs_out = [network.tx.TxOut(credit_tx.txs_out[0].coin_value, b'')]
    spend_tx = network.tx(1, txs_in, txs_out, unspents=credit_tx.tx_outs_as_spendable())
    return spend_tx


def dump_failure_info(spend_tx, script_in, script_out, flags, flags_string, expected, actual, message, comment):
    # return
    print()
    print(flags_string)
    print("EXPECTED: %s" % expected)
    print("ACTUAL: %s" % actual)
    print("MESSAGE: %s" % message)
    print(comment)
    print(network.script.disassemble(network.script.compile(script_in)))
    print(network.script.disassemble(network.script.compile(script_out)))

    def tbf(*args):
        opcode, data, pc, vm = args
        stack = vm.stack
        altstack = vm.altstack
        opd = network.script.disassemble_for_opcode_data(opcode, data)
        if len(altstack) == 0:
            altstack = ''
        print("%s %s\n  %3x  %s" % (stack, altstack, pc, opd))
        import pdb
        pdb.set_trace()
    print("test failed: '%s' '%s' : %s  %s" % (script_in, script_out, comment, flags_string))
    try:
        import pdb
        pdb.set_trace()
        spend_tx.check_solution(tx_in_idx=0, traceback_f=tbf, flags=flags)
    except Exception as ex:
        print(ex)
    try:
        spend_tx.check_solution(tx_in_idx=0, traceback_f=tbf, flags=flags)
    except Exception as ex:
        print(ex)
        import pdb
        pdb.set_trace()


def make_script_test(script_in, script_out, flags_string, comment, expected, coin_value, script_witness):
    script_in_bin = network.script.compile(script_in)
    script_out_bin = network.script.compile(script_out)
    script_witness_bin = [h2b(w) for w in script_witness]
    flags = parse_flags(flags_string)

    def f(self):
        try:
            credit_tx = build_credit_tx(script_out_bin, coin_value)
            spend_tx = build_spending_tx(script_in_bin, credit_tx)
            spend_tx.txs_in[0].witness = script_witness_bin
            msg = ''
            spend_tx.check_solution(tx_in_idx=0, flags=flags)
            r = 0
        except ScriptError as se:
            r = se.error_code()
            msg = se.args[0]
        except Exception:
            r = -1
        # for now, just deal with 0 versus nonzero
        expect_error = getattr(errno, expected)
        if r != expect_error:
            dump_failure_info(spend_tx, script_in, script_out, flags, flags_string, expected, r, msg, comment)
        self.assertEqual(r, expect_error)
    return f


def items_from_json(path):
    with open(path, "r") as f:
        for i in json.load(f):
            if len(i) >= 4:
                yield i


def inject():
    for idx, args in enumerate(items_from_json(SCRIPT_TESTS_JSON)):
        script_witness, coin_value = [], 0
        if type(args[0]) is list:
            script_witness, coin_value = args[0][:-1], int(1e8 * args[0][-1] + 0.5)
            args = args[1:]
        (script_in, script_out, flags, expected) = args[:4]
        comments = '/'.join(args[4:])
        name_of_f = "test_scripts_%03d" % idx
        setattr(TestTx, name_of_f,
                make_script_test(script_in, script_out, flags, comments, expected, coin_value, script_witness))
        print("adding %s" % name_of_f)


inject()



import unittest

from pycoin.symbols.btc import network as BTC
from pycoin.symbols.xtn import network as XTN


class KeyParserTest(unittest.TestCase):

    def test_parse_bip32_prv(self):
        key = BTC.parse("xprv9s21ZrQH143K31AgNK5pyVvW23gHnkBq2wh5aEk6g1s496M8ZMjx"
                        "ncCKZKgb5jZoY5eSJMJ2Vbyvi2hbmQnCuHBujZ2WXGTux1X2k9Krdtq")
        self.assertEqual(
            key.secret_exponent(), 0x91880b0e3017ba586b735fe7d04f1790f3c46b818a2151fb2def5f14dd2fd9c3)
        self.assertEqual(key.address(), "19Vqc8uLTfUonmxUEZac7fz1M5c5ZZbAii")
        self.assertEqual(key.address(is_compressed=False), "1MwkRkogzBRMehBntgcq2aJhXCXStJTXHT")
        subkey = key.subkey_for_path("0")
        self.assertEqual(subkey.address(), "1NV3j6NgeAkWBytXiQkWxMFLBtTdbef1rp")

    def test_parse_bip32_prv_xtn(self):
        key = XTN.parse("tprv8ZgxMBicQKsPdpQD2swL99YVLB6W2GDqNVcCSfAZ9zMXvh6DYj5iJMZmUVrF66"
                        "x7uXBDJSunexZjAtFLtd89iLTWGCEpBdBxs7GTBnEksxV")
        self.assertEqual(
            key.secret_exponent(), 0x91880b0e3017ba586b735fe7d04f1790f3c46b818a2151fb2def5f14dd2fd9c3)
        self.assertEqual(key.address(), "mp1nuBzKGgv4ZtS5x8YywbCLD5CnVfT7hV")
        self.assertEqual(key.address(is_compressed=False), "n2ThiotfoCrcRofQcFbCrVX2PC89s2KUjh")
        subkey = key.subkey_for_path("0")
        self.assertEqual(subkey.address(), "n31129TfTCBky6N9RyitnGTf3t4LYwCV6A")

    def test_parse_bip32_pub(self):
        key = BTC.parse("xpub661MyMwAqRbcFVF9ULcqLdsEa5WnCCugQAcgNd9iEMQ31tgH6u4"
                        "DLQWoQayvtSVYFvXz2vPPpbXE1qpjoUFidhjFj82pVShWu9curWmb2zy")
        self.assertEqual(key.secret_exponent(), None)
        self.assertEqual(key.address(), "19Vqc8uLTfUonmxUEZac7fz1M5c5ZZbAii")
        self.assertEqual(key.address(is_compressed=False), "1MwkRkogzBRMehBntgcq2aJhXCXStJTXHT")
        subkey = key.subkey_for_path("0")
        self.assertEqual(subkey.address(), "1NV3j6NgeAkWBytXiQkWxMFLBtTdbef1rp")

    def test_parse_bad_bip32_prv(self):
        key = BTC.parse("xprv9s21ZrQH143K31AgNK5pyVvW23gHnkBq2wh5aEk6g1s496M8ZMjx"
                        "ncCKZKgb5jZoY5eSJMJ2Vbyvi2hbmQnCuHBujZ2WXGTux1X2k9Krdtr")
        self.assertEqual(key, None)

    def test_parse_wif(self):
        key = BTC.parse("KwDiBf89QgGbjEhKnhXJuH7LrciVrZi3qYjgd9M7rFU73sVHnoWn")
        self.assertEqual(key.secret_exponent(), 1)

    def test_parse_bad_wif(self):
        key = BTC.parse("KwDiBf89QgGbjEhKnhXJuH7LrciVrZi3qYjgd9M7rFU73sVHnoWo")
        self.assertEqual(key, None)

    def test_parse_address(self):
        key = BTC.parse("1BgGZ9tcN4rm9KBzDn7KprQz87SZ26SAMH")
        self.assertEqual(key.address(), "1BgGZ9tcN4rm9KBzDn7KprQz87SZ26SAMH")

    def test_parse_bad_address(self):
        key = BTC.parse("1BgGZ9tcN4rm9KBzDn7KprQz87SZ26SAMW")
        self.assertEqual(key, None)

    def test_parse_electrum_seed(self):
        key = BTC.parse("E:00000000000000000000000000000001")
        self.assertEqual(
            key.secret_exponent(), 0x2ccdb632d4630c8e5a417858f70876afe5585c15b1c0940771af9ac160201b1d)
        self.assertEqual(key.address(), "16e8FARWaEo7Cf2rYxzr8Lg3S8JP2dwBxh")
        subkey = key.subkey("1")
        self.assertEqual(subkey.wif(), "5KYqyRxoMGnwsXfEFWtVifAKTzU9RcAZu1hme6GLMECKdWHybns")

    def test_parse_electrum_master_private(self):
        key = BTC.parse("E:0000000000000000000000000000000000000000000000000000000000000001")
        self.assertEqual(key.secret_exponent(), 1)




import os
import threading

from .registry import network_for_netcode

THREAD_LOCALS = threading.local()


def check_netcode(netcode):
    if network_for_netcode(netcode) is None:
        raise ValueError("unknown netcode %s" % netcode)


def _netcode_for_env():
    p = os.getenv("PYCOIN_DEFAULT_NETCODE")
    if p is None:
        p = 'BTC'
    check_netcode(p)
    return p


DEFAULT_NETCODE = _netcode_for_env()


def set_default_netcode(netcode):
    global DEFAULT_NETCODE
    check_netcode(netcode)
    DEFAULT_NETCODE = netcode


def set_default_netcode_for_thread(netcode):
    check_netcode(netcode)
    THREAD_LOCALS.netcode = netcode


def get_current_netcode():
    # check the thread local first
    # if that doesn't exist, use the global default
    return getattr(THREAD_LOCALS, "netcode", DEFAULT_NETCODE)


def get_current_network():
    return network_for_netcode(get_current_netcode())



from pycoin.networks.bitcoinish import create_bitcoinish_network


network = create_bitcoinish_network(
    network_name="Monacoin", symbol="MONA", subnet_name="mainnet",
    wif_prefix_hex="b0", sec_prefix="MONASEC:", address_prefix_hex="32", pay_to_script_prefix_hex="37",
    bip32_prv_prefix_hex="0488ade4", bip32_pub_prefix_hex="0488b21e", bech32_hrp="mona",
    magic_header_hex="fbc0b6db", default_port=9401,
    dns_bootstrap=["dnsseed.monacoin.org"])



import functools
import pdb


@functools.total_ordering
class Atom(object):
    def __init__(self, name):
        self.name = name

    def dependencies(self):
        return frozenset([self])

    def __len__(self):
        # HACK to allow MAX_BLOB_LENGTH comparison to succeed
        return 0

    def __eq__(self, other):
        if isinstance(other, Atom):
            return self.name == other.name
        return False

    def __lt__(self, other):
        if isinstance(other, Atom):
            return self.name < other.name
        return False

    def __hash__(self):
        return self.name.__hash__()

    def __repr__(self):
        return "<%s>" % self.name


class Operator(Atom):
    def __init__(self, op_name, *args):
        self._op_name = op_name
        self._args = tuple(args)
        s = set()
        for a in self._args:
            if hasattr(a, "dependencies"):
                s.update(a.dependencies())
        self._dependencies = frozenset(s)

    def __hash__(self):
        return self._args.__hash__()

    def __eq__(self, other):
        if isinstance(other, Operator):
            return self._op_name, self._args == other._op_name, other._args
        return False

    def dependencies(self):
        return self._dependencies

    def __repr__(self):
        return "(%s %s)" % (self._op_name, ' '.join(repr(a) for a in self._args))


def make_op_if(constraints):

    def my_op_if(vm):
        pdb.set_trace()
        t = vm.stack.pop()
        t = Operator('IF', t)
        vm.stack.append(t)
    return my_op_if


def make_op_hash160(constraints):

    def my_op_hash160(vm):
        t = vm.stack.pop()
        t = Operator('HASH160', t)
        vm.stack.append(t)
    my_op_hash160.stack_size = 1
    return my_op_hash160


def make_op_equal(constraints):
    def my_op_equal(vm):
        t1 = vm.stack.pop()
        t2 = vm.stack.pop()
        c = Operator('EQUAL', t1, t2)
        vm.append(c)
    my_op_equal.stack_size = 2
    return my_op_equal


def make_op_equalverify(constraints):
    def my_op_equalverify(vm):
        t1 = vm.stack.pop()
        t2 = vm.stack.pop()
        c = Operator('EQUAL', t1, t2)
        constraints.append(c)
    my_op_equalverify.stack_size = 2
    return my_op_equalverify


def make_op_checksig(constraints):
    def my_op_checksig(vm):

        def sighash_f(signature_type):
            return vm.signature_for_hash_type_f(signature_type, [], vm)

        t1 = vm.stack.pop()
        t2 = vm.stack.pop()
        t = Operator('SIGNATURES_CORRECT', [t1], [t2], sighash_f)
        constraints.append(Operator('IS_PUBKEY', t1))
        constraints.append(Operator('IS_SIGNATURE', t2))
        vm.stack.append(t)
    return my_op_checksig


def make_op_checkmultisig(constraints):
    def my_op_checkmultisig(vm):

        def sighash_f(signature_type):
            return vm.signature_for_hash_type_f(signature_type, [], vm)

        key_count = vm.IntStreamer.int_from_script_bytes(vm.stack.pop(), require_minimal=False)
        public_pair_blobs = []
        for i in range(key_count):
            constraints.append(Operator('IS_PUBKEY', vm.stack[-1]))
            public_pair_blobs.append(vm.stack.pop())
        signature_count = vm.IntStreamer.int_from_script_bytes(vm.stack.pop(), require_minimal=False)
        sig_blobs = []
        for i in range(signature_count):
            constraints.append(Operator('IS_SIGNATURE', vm.stack[-1]))
            sig_blobs.append(vm.stack.pop())
        t1 = vm.stack.pop()
        constraints.append(Operator('EQUAL', t1, b''))
        t = Operator('SIGNATURES_CORRECT', public_pair_blobs, sig_blobs, sighash_f)
        vm.stack.append(t)
    return my_op_checkmultisig


def make_traceback_f(constraints, int_for_opcode_f, reset_stack_f):

    TWEAKED_OPCODES = (
        ("OP_HASH160", make_op_hash160),
        ("OP_EQUALVERIFY", make_op_equalverify),
        ("OP_EQUAL", make_op_equal),
        ("OP_CHECKSIG", make_op_checksig),
        ("OP_CHECKMULTISIG", make_op_checkmultisig),
    )

    MY_OPCODES = {int_for_opcode_f(k): v(constraints) for k, v in TWEAKED_OPCODES}

    def prelaunch(vmc):
        if not vmc.is_solution_script:
            # reset stack
            vmc.stack = reset_stack_f(vmc.stack)

    def traceback_f(opcode, data, pc, vm):
        f = MY_OPCODES.get(opcode)
        if f is None:
            return
        stack_size = getattr(f, "stack_size", 0)
        if stack_size and all(not isinstance(v, Atom) for v in vm.stack[-stack_size:]):
            return
        return f

    def postscript(vmc):
        if not vmc.is_solution_script:
            if isinstance(vmc.stack[-1], Atom):
                constraints.append(vmc.stack[-1])
            vmc.stack = [vmc.VM_TRUE]

    traceback_f.prelaunch = prelaunch
    traceback_f.postscript = postscript
    return traceback_f



import json
import io

from .agent import urlopen

from pycoin.coins.bitcoin.Tx import Tx
from pycoin.encoding.hexbytes import b2h_rev, h2b, h2b_rev
from pycoin.networks.default import get_current_netcode


class BlockcypherProvider(object):
    def __init__(self, api_key="", netcode=None):
        NETWORK_PATHS = {
            "BTC": "btc/main",
            "XTN": "btc/test3",
            "DASH": "dash/main",
        }

        if netcode is None:
            netcode = get_current_netcode()

        self.network_path = NETWORK_PATHS.get(netcode)
        self.api_key = api_key

    def base_url(self, args):
        return "https://api.blockcypher.com/v1/%s/%s" % (self.network_path, args)

    def spendables_for_address(self, address):
        """
        Return a list of Spendable objects for the
        given bitcoin address.
        """
        spendables = []
        url_append = "?unspentOnly=true&token=%s&includeScript=true" % self.api_key
        url = self.base_url("addrs/%s%s" % (address, url_append))
        result = json.loads(urlopen(url).read().decode("utf8"))
        for txn in result.get("txrefs", []):
            coin_value = txn.get("value")
            script = h2b(txn.get("script"))
            previous_hash = h2b_rev(txn.get("tx_hash"))
            previous_index = txn.get("tx_output_n")
            spendables.append(Tx.Spendable(coin_value, script, previous_hash, previous_index))
        return spendables

    def tx_for_tx_hash(self, tx_hash):
        """
        returns the pycoin.tx object for tx_hash
        """
        try:
            url_append = "?token=%s&includeHex=true" % self.api_key
            url = self.base_url("txs/%s%s" % (b2h_rev(tx_hash), url_append))
            result = json.loads(urlopen(url).read().decode("utf8"))
            tx = Tx.parse(io.BytesIO(h2b(result.get("hex"))))
            return tx
        except Exception:
            raise Exception

    def get_balance(self, address):
        """
        returns the balance object from blockcypher for address
        """
        url_append = "/balance?token=%s" % self.api_key
        url = self.base_url("addrs/%s" % (address + url_append))
        result = json.loads(urlopen(url).read().decode("utf8"))
        return result

    def broadcast_tx(self, tx):
        """
        broadcast a transaction to the network
        """
        url = self.base_url("txs/push")
        data = {"tx": tx.as_hex()}
        result = json.loads(urlopen(url, data=json.dumps(data).encode("utf8")).read().decode("utf8"))
        return result




from .SolutionChecker import BcashSolutionChecker
from .Solver import BcashSolver

from pycoin.coins.bitcoin.Tx import Tx as BaseTx


class Tx(BaseTx):
    Solver = BcashSolver
    SolutionChecker = BcashSolutionChecker



from pycoin.encoding.hexbytes import b2h


class Contract(object):
    """
    A script that encumbers coins.
    """
    def __init__(self, script_info, network):
        self._script_info = script_info
        self._network = network

    def info(self):
        return self._script_info

    def hash160(self):
        """
        Return a 20-byte hash corresponding to this script (or None if not applicable).
        """
        return self._script_info.get("hash160")

    def address(self):
        """
        Return a string with the address for this script (or None if this script does
        have a corresponding address).
        """
        return self._network.address.for_script_info(self._script_info)

    def script(self):
        """
        Return a :class:`bytes <bytes>` with a binary image of the script.
        """
        return self._network.contract.for_info(self._script_info)

    def disassemble(self):
        """
        Return a text string of the disassembly of the script.
        """
        return self._network.script.disassemble(self.script())

    def output(self):
        """
        Return a 20-byte hash corresponding to this script (or None if not applicable).
        """
        hash160 = self._script_info.get("hash160", None)
        if hash160:
            yield ("hash160", b2h(hash160), None)

        address = self.address()
        yield ("address", address, "%s address" % self._network.network_name)
        yield ("%s_address" % self._network.symbol, address, "legacy")

    def __repr__(self):
        return "<%s>" % self.address()




if hasattr(int, "to_bytes"):
    def to_bytes_32(v):
        return v.to_bytes(32, byteorder="big")

    def from_bytes_32(v):
        return int.from_bytes(v, byteorder="big")
else:
    from .base_conversion import from_long, to_long
    from ..intbytes import byte2int

    def to_bytes_32(v):
        v = from_long(v, 0, 256, lambda x: x)
        if len(v) > 32:
            raise ValueError("input to to_bytes_32 is too large")
        return ((b'\0' * 32) + v)[-32:]

    def from_bytes_32(v):
        if len(v) > 32:
            raise OverflowError("int too big to convert")
        return to_long(256, byte2int, v)[0]


"""
Various utilities useful for converting one Bitcoin format to another, including some
the human-transcribable format hashed_base58.


The MIT License (MIT)

Copyright (c) 2013 by Richard Kiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""



import hashlib
import struct
import unittest

from pycoin.coins.exceptions import BadSpendableError
from pycoin.symbols.btc import network


Spendable = network.tx.Spendable

BITCOIN_ADDRESSES = [network.keys.private(i).address() for i in range(1, 21)]

WIFS = [network.keys.private(i).wif() for i in range(1, 21)]

FAKE_HASH = hashlib.sha256(struct.pack("Q", 1)).digest()


class SpendTest(unittest.TestCase):

    def test_simple_spend(self):

        FEE = 10000

        # create a fake Spendable
        COIN_VALUE = 100000000
        spendables = [Spendable(COIN_VALUE, network.contract.for_address(BITCOIN_ADDRESSES[0]), FAKE_HASH, 0)]

        EXPECTED_IDS = [
            "d28bff6c4a8a0f9e7d5b7df0670d07b43c5613d8c9b14e84707b1e2c0154a978",
            "7afbe63b00171b18f806ebd48190ebc1c68cadf286a85489c06ebe43d146489e",
            "2b90c150ba1d080a0816952f5d9c2642d408989cbc4d4c540591c8c9241294bd",
            "17b0b5b22887081595c1a9ad153e903f63bb8682ae59d6082df018dc617e5e67",
            "dff1b34c243becb096ad2a2d6119973067a8137cc8bf95615e742bbf6f0944c1",
            "206bbfbb759a8f91901d86b62390d7587f6097a32994ece7752d143fc8a02cee",
            "7841412716ad35cbc9954e547ba85be89e5ed0b34ed5fb8d7594517318dc10d6",
            "8b7e643bf47db46ada7a75b8498990b111fe20917b5610ca6759b8b0078ccd5e",
            "5756f0a6d5a2bbb93a07f0729d3773aaafd21393ede3ec0e20b0b5219ca45548",
            "32dcbb34965ea72d2caa59eb1e907aa28bac2afea43214c1809f5d8ed360f30e",
        ]

        for count in range(1, 11):
            tx = network.tx_utils.create_signed_tx(spendables, BITCOIN_ADDRESSES[1:count+1], wifs=WIFS[:1])
            self.assertEqual(tx.bad_solution_count(), 0)
            self.assertEqual(tx.fee(), FEE)
            self.assertEqual(tx.id(), EXPECTED_IDS[count-1])
            for idx in range(1, count+1):
                script = tx.txs_out[idx-1].puzzle_script()
                address = network.address.for_script(script)
                self.assertEqual(address, BITCOIN_ADDRESSES[idx])
            # TODO: add check that s + s < generator for each signature
            for i in range(count):
                extra = (1 if i < ((COIN_VALUE - FEE) % count) else 0)
                self.assertEqual(tx.txs_out[i].coin_value, (COIN_VALUE - FEE) // count + extra)

    def test_confirm_input(self):
        # create a fake Spendable
        COIN_VALUE = 100000000
        spendables = [Spendable(COIN_VALUE, network.contract.for_address(BITCOIN_ADDRESSES[0]), FAKE_HASH, 0)]

        tx_1 = network.tx_utils.create_signed_tx(spendables, BITCOIN_ADDRESSES[1:2], wifs=WIFS[:1])

        spendables = tx_1.tx_outs_as_spendable()

        tx_db = dict((tx.hash(), tx) for tx in [tx_1])

        tx_2 = network.tx_utils.create_signed_tx(spendables, BITCOIN_ADDRESSES[2:3], wifs=WIFS[:3])
        tx_2.validate_unspents(tx_db)

        tx_2 = network.tx_utils.create_signed_tx(
            [s.as_dict() for s in spendables], BITCOIN_ADDRESSES[2:3], wifs=WIFS[:3])
        tx_2.validate_unspents(tx_db)

        tx_2 = network.tx_utils.create_signed_tx(
            [s.as_text() for s in spendables], BITCOIN_ADDRESSES[2:3], wifs=WIFS[:3])
        tx_2.validate_unspents(tx_db)

    def test_confirm_input_raises(self):
        # create a fake Spendable
        COIN_VALUE = 100000000
        spendables = [Spendable(COIN_VALUE, network.contract.for_address(BITCOIN_ADDRESSES[0]), FAKE_HASH, 0)]

        tx_1 = network.tx_utils.create_signed_tx(spendables, BITCOIN_ADDRESSES[1:2], wifs=WIFS[:1])
        spendables = tx_1.tx_outs_as_spendable()
        spendables[0].coin_value += 100

        tx_db = dict((tx.hash(), tx) for tx in [tx_1])
        tx_2 = network.tx_utils.create_signed_tx(spendables, BITCOIN_ADDRESSES[2:3], wifs=WIFS[:3])

        self.assertRaises(BadSpendableError, tx_2.validate_unspents, tx_db)


if __name__ == "__main__":
    unittest.main()



from pycoin.coins.bcash.Tx import Tx as BcashTx
from pycoin.networks.bitcoinish import create_bitcoinish_network


network = create_bitcoinish_network(
    symbol="BCH", network_name="Bcash", subnet_name="mainnet", tx=BcashTx,
    wif_prefix_hex="80", sec_prefix="BCHSEC:", address_prefix_hex="00", pay_to_script_prefix_hex="05",
    bip32_prv_prefix_hex="0488ade4", bip32_pub_prefix_hex="0488B21E",
    magic_header_hex="F9BEB4D9", default_port=8333,
    dns_bootstrap=[
        "seed.bitcoinabc.org", "seed-abc.bitcoinforks.org",
        "btccash-seeder.bitcoinunlimited.info", "seed.bitprim.org",
        "seed.deadalnix.me", "seeder.criptolayer.net"
    ])



import hashlib

from .subpaths import subpaths_for_path_range

from pycoin.encoding.bytes32 import from_bytes_32, to_bytes_32
from pycoin.encoding.hash import double_sha256
from pycoin.encoding.hexbytes import b2h
from pycoin.key.Key import Key


def initial_key_to_master_key(initial_key):
    """
    initial_key:
        a hex string of length 32
    """
    b = initial_key.encode("utf8")
    orig_input = b
    for i in range(100000):
        b = hashlib.sha256(b + orig_input).digest()
    return from_bytes_32(b)


class ElectrumWallet(Key):
    def __init__(self, initial_key=None, master_private_key=None, public_pair=None, master_public_key=None):
        if [initial_key, public_pair, master_private_key, master_public_key].count(None) != 3:
            raise ValueError(
                "exactly one of initial_key, master_private_key, master_public_key must be non-None")
        self._initial_key = initial_key

        if initial_key is not None:
            master_private_key = initial_key_to_master_key(initial_key)
        if master_public_key:
            public_pair = tuple(from_bytes_32(master_public_key[idx:idx+32]) for idx in (0, 32))
        super(ElectrumWallet, self).__init__(
            secret_exponent=master_private_key, public_pair=public_pair, is_compressed=False)

    @classmethod
    def deserialize(class_, blob):
        if len(blob) == 32:
            return class_(master_private_key=from_bytes_32(blob))
        if len(blob) == 64:
            return class_(master_public_key=blob)

    def serialize(self):
        if self._secret_exponent:
            return to_bytes_32(self._secret_exponent)
        return self.master_public_key()

    def secret_exponent(self):
        if self._secret_exponent is None and self._initial_key:
            self._secret_exponent = initial_key_to_master_key(b2h(self._initial_key))
        return self._secret_exponent

    def master_private_key(self):
        return self.secret_exponent()

    def master_public_key(self):
        return self.sec()[1:]

    def public_copy(self):
        if self.secret_exponent() is None:
            return self
        return self.__class__(public_pair=self.public_pair())

    def subkey_for_path(self, path):
        return self.subkey(path)

    def subkey(self, path):
        """
        path:
            of the form "K" where K is an integer index, or "K/N" where N is usually
            a 0 (deposit address) or 1 (change address)
        """
        t = path.split("/")
        if len(t) == 2:
            n, for_change = t
        else:
            n, = t
            for_change = 0
        b = (str(n) + ':' + str(for_change) + ':').encode("utf8") + self.master_public_key()
        offset = from_bytes_32(double_sha256(b))
        if self.secret_exponent():
            return self.__class__(
                master_private_key=((self.master_private_key() + offset) % self._generator.order())
            )
        p1 = offset * self._generator
        x, y = self.public_pair()
        p2 = self._generator.Point(x, y)
        p = p1 + p2
        return self.__class__(public_pair=p)

    def subkeys(self, path):
        """
        A generalized form that can return multiple subkeys.
        """
        for _ in subpaths_for_path_range(path, hardening_chars="'pH"):
            yield self.subkey(_)

    def __repr__(self):
        return "Electrum<E:%s>" % b2h(self.master_public_key())



import hashlib
import unittest

from pycoin.ecdsa.intstream import from_bytes
from pycoin.ecdsa.rfc6979 import deterministic_generate_k


class RFC6979TestCase(unittest.TestCase):

    def test_deterministic_generate_k_A_1(self):
        """
        The example in http://tools.ietf.org/html/rfc6979#appendix-A.1
        """
        h = hashlib.sha256(b'sample').digest()
        val = from_bytes(h)
        self.assertEqual(val, 0xAF2BDBE1AA9B6EC1E2ADE1D694F41FC71A831D0268E9891562113D8A62ADD1BF)
        q = 0x4000000000000000000020108A2E0CC0D99F8A5EF
        x = 0x09A4D6792295A7F730FC3F2B49CBC0F62E862272F
        k = deterministic_generate_k(q, x, val)
        self.assertEqual(k, 0x23AF4074C90A02B3FE61D286D5C87F425E6BDD81B)

    def test_deterministic_generate_k_A_2_1(self):
        """
        The example in https://tools.ietf.org/html/rfc6979#appendix-A.2.3
        """
        hashes_values = (
            (hashlib.sha1, 0x37D7CA00D2C7B0E5E412AC03BD44BA837FDD5B28CD3B0021),
            (hashlib.sha224, 0x4381526B3FC1E7128F202E194505592F01D5FF4C5AF015D8),
            (hashlib.sha256, 0x32B1B6D7D42A05CB449065727A84804FB1A3E34D8F261496),
            (hashlib.sha384, 0x4730005C4FCB01834C063A7B6760096DBE284B8252EF4311),
            (hashlib.sha512, 0xA2AC7AB055E4F20692D49209544C203A7D1F2C0BFBC75DB1),
            )
        q = 0xFFFFFFFFFFFFFFFFFFFFFFFF99DEF836146BC9B1B4D22831
        x = 0x6FAB034934E4C0FC9AE67F5B5659A9D7D1FEFD187EE09FD4
        for h, v in hashes_values:
            v_sample = from_bytes(h(b'sample').digest())
            k = deterministic_generate_k(q, x, v_sample, h)
            self.assertEqual(k, v)

        hashes_values = (
            (hashlib.sha1, 0xD9CF9C3D3297D3260773A1DA7418DB5537AB8DD93DE7FA25),
            (hashlib.sha224, 0xF5DC805F76EF851800700CCE82E7B98D8911B7D510059FBE),
            (hashlib.sha256, 0x5C4CE89CF56D9E7C77C8585339B006B97B5F0680B4306C6C),
            (hashlib.sha384, 0x5AFEFB5D3393261B828DB6C91FBC68C230727B030C975693),
            (hashlib.sha512, 0x0758753A5254759C7CFBAD2E2D9B0792EEE44136C9480527),
            )
        for h, v in hashes_values:
            v_sample = from_bytes(h(b'test').digest())
            k = deterministic_generate_k(q, x, v_sample, h)
            self.assertEqual(k, v)

    def test_deterministic_generate_k_A_2_5(self):
        """
        The example in https://tools.ietf.org/html/rfc6979#appendix-A.2.5
        """
        h = hashlib.sha256(b'sample').digest()
        val = from_bytes(h)
        self.assertEqual(val, 0xAF2BDBE1AA9B6EC1E2ADE1D694F41FC71A831D0268E9891562113D8A62ADD1BF)
        generator_order = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFF16A2E0B8F03E13DD29455C5C2A3D
        secret_exponent = 0xF220266E1105BFE3083E03EC7A3A654651F45E37167E88600BF257C1
        k = deterministic_generate_k(generator_order, secret_exponent, val)
        self.assertEqual(k, 0xAD3029E0278F80643DE33917CE6908C70A8FF50A411F06E41DEDFCDC)


if __name__ == '__main__':
    unittest.main()



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="AXE", network_name="Axe", subnet_name="mainnet",
    wif_prefix_hex="cc", address_prefix_hex="37", pay_to_script_prefix_hex="10",
    bip32_prv_prefix_hex="0488ade4", bip32_pub_prefix_hex="0488b21e")



import ctypes.util
import os
import platform

from .bignum import bignum_type_for_library


NID_X9_62_prime256v1 = 415
NID_secp256k1 = 714


class BignumContext(ctypes.Structure):
    pass


def set_api(library, api_info):
    for f_name, argtypes, restype in api_info:
        f = getattr(library, f_name)
        f.argtypes = argtypes
        f.restype = restype


def load_library():
    system = platform.system()
    PYCOIN_LIBCRYPTO_PATH = os.getenv("PYCOIN_LIBCRYPTO_PATH")

    if PYCOIN_LIBCRYPTO_PATH:
        library_path = PYCOIN_LIBCRYPTO_PATH
    elif system == 'Windows':
        if platform.architecture()[0] == '64bit':
            library_path = ctypes.util.find_library('libeay64')
        else:
            library_path = ctypes.util.find_library('libeay32')

    else:
        # on Mac OS 10.15.1 trying to load "libcrypto" crashes
        # but crypto.0.9.8 works, so try to load that one first
        for p in ["crypto.0.9.8", "crypto"]:
            library_path = ctypes.util.find_library(p)
            if library_path:
                break

    if library_path is None:
        return None

    library = ctypes.CDLL(library_path)

    library.BignumType = bignum_type_for_library(library)

    BN_P = ctypes.POINTER(library.BignumType)
    BN_CTX = ctypes.POINTER(BignumContext)

    BIGNUM_API = [
        ("BN_new", [], BN_P),
        ("BN_set_word", [BN_P, ctypes.c_ulong], ctypes.c_int),
        ("BN_clear_free", [BN_P], None),
        ("BN_bin2bn", [ctypes.c_char_p, ctypes.c_int, BN_P], BN_P),
        ("BN_mod_inverse", [BN_P, BN_P, BN_P, BN_CTX], BN_P),
        ("BN_CTX_new", [], BN_CTX),
        ("BN_CTX_free", [BN_CTX], None),
        ("BN_mpi2bn", [ctypes.c_char_p, ctypes.c_int, BN_P], BN_P),
    ]

    ECC_API = [
        ("EC_GROUP_new_by_curve_name", [ctypes.c_int], ctypes.c_void_p),
        ("EC_POINT_new", [ctypes.c_void_p], ctypes.c_void_p),  # TODO: make this a EC_POINT type
        ("EC_POINT_free", [ctypes.c_void_p], None),
        ("EC_POINT_set_affine_coordinates_GFp",
            [ctypes.c_void_p, ctypes.c_void_p, BN_P, BN_P, BN_CTX], ctypes.c_int),
        ("EC_POINT_get_affine_coordinates_GFp",
            [ctypes.c_void_p, ctypes.c_void_p, BN_P, BN_P, BN_CTX], ctypes.c_int),
        ("EC_POINT_mul",
            [ctypes.c_void_p, ctypes.c_void_p, BN_P, ctypes.c_void_p, BN_P, BN_CTX], ctypes.c_int),
    ]
    set_api(library, BIGNUM_API)
    set_api(library, ECC_API)
    return library


OpenSSL = load_library()


def create_OpenSSLOptimizations(curve_id):

    class noop:
        pass

    native = os.getenv("PYCOIN_NATIVE")
    if native and native.lower() != "openssl":
        return noop

    if not OpenSSL:
        return noop

    class Optimizations:

        if OpenSSL:
            openssl_group = OpenSSL.EC_GROUP_new_by_curve_name(curve_id)

        def multiply(self, p, e):
            "Use OpenSSL to perform point mulitiplication."
            if e == 0 or p == self._infinity:
                return self._infinity

            bn_x = OpenSSL.BignumType(p[0])
            bn_y = OpenSSL.BignumType(p[1])
            bn_n = OpenSSL.BignumType(e)

            ctx = OpenSSL.BN_CTX_new()
            ec_result = OpenSSL.EC_POINT_new(self.openssl_group)
            ec_point = OpenSSL.EC_POINT_new(self.openssl_group)

            OpenSSL.EC_POINT_set_affine_coordinates_GFp(self.openssl_group, ec_point, bn_x, bn_y, ctx)

            OpenSSL.EC_POINT_mul(self.openssl_group, ec_result, None, ec_point, bn_n, ctx)

            OpenSSL.EC_POINT_get_affine_coordinates_GFp(self.openssl_group, ec_result, bn_x, bn_y, ctx)
            OpenSSL.EC_POINT_free(ec_point)
            OpenSSL.EC_POINT_free(ec_result)
            OpenSSL.BN_CTX_free(ctx)
            return self.Point(bn_x.to_int(), bn_y.to_int())

        def raw_mul(self, e):
            """Multiply the generator by an integer."""
            return self.multiply(self, e)

        def inverse_mod(self, a, p):
            ctx = OpenSSL.BN_CTX_new()
            a1 = OpenSSL.BignumType(a)
            OpenSSL.BN_mod_inverse(a1, a1, OpenSSL.BignumType(p), ctx)
            OpenSSL.BN_CTX_free(ctx)
            return a1.to_int()

    return Optimizations



from pycoin.networks.bitcoinish import create_bitcoinish_network

network = create_bitcoinish_network(
    symbol="PIVX", network_name="PIVX", subnet_name="mainnet",
    wif_prefix_hex="d4", address_prefix_hex="1e", pay_to_script_prefix_hex="0064",
    bip32_prv_prefix_hex="0221312b", bip32_pub_prefix_hex="022d2533")



import binascii
import unittest

from pycoin.encoding.hexbytes import h2b
from pycoin.symbols.btc import network


class ValidationTest(unittest.TestCase):

    def setUp(self):
        self._key = network.keys.private(1)

    def test_validate_multisig_tx(self):
        # this is a transaction in the block chain
        # the unspents are included too, so it can be validated
        TX_HEX = (
            "01000000025718fb915fb8b3a802bb699ddf04dd91261ef6715f5f2820a2b1b9b7e38b"
            "4f27000000004a004830450221008c2107ed4e026ab4319a591e8d9ec37719cdea0539"
            "51c660566e3a3399428af502202ecd823d5f74a77cc2159d8af2d3ea5d36a702fef9a7"
            "edaaf562aef22ac35da401ffffffff038f52231b994efb980382e4d804efeadaee13cf"
            "e01abe0d969038ccb45ec17000000000490047304402200487cd787fde9b337ab87f9f"
            "e54b9fd46d5d1692aa58e97147a4fe757f6f944202203cbcfb9c0fc4e3c453938bbea9"
            "e5ae64030cf7a97fafaf460ea2cb54ed5651b501ffffffff0100093d00000000001976"
            "a9144dc39248253538b93d3a0eb122d16882b998145888ac0000000002000000000000"
            "004751210351efb6e91a31221652105d032a2508275f374cea63939ad72f1b1e02f477"
            "da782100f2b7816db49d55d24df7bdffdbc1e203b424e8cd39f5651ab938e5e4a19356"
            "9e52ae404b4c00000000004751210351efb6e91a31221652105d032a2508275f374cea"
            "63939ad72f1b1e02f477da7821004f0331742bbc917ba2056a3b8a857ea47ec088dd10"
            "475ea311302112c9d24a7152ae")
        tx = network.tx.from_hex(TX_HEX)
        self.assertEqual(tx.id(), "70c4e749f2b8b907875d1483ae43e8a6790b0c8397bbb33682e3602617f9a77a")
        self.assertEqual(tx.bad_solution_count(), 0)

    def test_validate_block_data(self):
        # block 80971
        block_80971_id = '00000000001126456C67A1F5F0FF0268F53B4F22E0531DC70C7B69746AF69DAC'.lower()
        block_80971_data = h2b(
            "01000000950A1631FB9FAC411DFB173487B9E18018B7C6F7147E78C062584100000000"
            "00A881352F97F14BF191B54915AE124E051B8FE6C3922C5082B34EAD503000FC34D891"
            "974CED66471B4016850A04010000000100000000000000000000000000000000000000"
            "00000000000000000000000000FFFFFFFF0804ED66471B02C301FFFFFFFF0100F2052A"
            "01000000434104CB6B6B4EADC96C7D08B21B29D0ADA5F29F9378978CABDB602B8B65DA"
            "08C8A93CAAB46F5ABD59889BAC704925942DD77A2116D10E0274CAD944C71D3D1A6705"
            "70AC0000000001000000018C55ED829F16A4E43902940D3D33005264606D5F7D555B5F"
            "67EE4C033390C2EB010000008A47304402202D1BF606648EDCDB124C1254930852D991"
            "88E1231715031CBEAEA80CCFD2B39A02201FA9D6EE7A1763580E342474FC1AEF59B046"
            "8F98479953437F525063E25675DE014104A01F763CFBF5E518C628939158AF3DC0CAAC"
            "35C4BA7BC1CE8B7E634E8CDC44E15F0296B250282BD649BAA8398D199F2424FCDCD88D"
            "3A9ED186E4FD3CB9BF57CFFFFFFFFF02404B4C00000000001976A9148156FF75BEF24B"
            "35ACCE3C05289A2411E1B0E57988AC00AA38DF010000001976A914BC7E692A5FFE95A5"
            "96712F5ED83393B3002E452E88AC0000000001000000019C97AFDF6C9A31FFA86D71EA"
            "79A079001E2B59EE408FD418498219400639AC0A010000008B4830450220363CFFAE09"
            "599397B21E6D8A8073FB1DFBE06B6ACDD0F2F7D3FEA86CA9C3F605022100FA255A6ED2"
            "3FD825C759EF1A885A31CAD0989606CA8A3A16657D50FE3CEF5828014104FF444BAC08"
            "308B9EC97F56A652AD8866E0BA804DA97868909999566CB377F4A2C8F1000E83B49686"
            "8F3A282E1A34DF78565B65C15C3FA21A0763FD81A3DFBBB6FFFFFFFF02C05EECDE0100"
            "00001976A914588554E6CC64E7343D77117DA7E01357A6111B7988AC404B4C00000000"
            "001976A914CA6EB218592F289999F13916EE32829AD587DBC588AC0000000001000000"
            "01BEF5C9225CB9FE3DEF929423FA36AAD9980B9D6F8F3070001ACF3A5FB389A69F0000"
            "00004A493046022100FB23B1E2F2FB8B96E04D220D385346290A9349F89BBBC5C225D5"
            "A56D931F8A8E022100F298EB28294B90C1BAF319DAB713E7CA721AAADD8FCC15F849DE"
            "7B0A6CF5412101FFFFFFFF0100F2052A010000001976A9146DDEA8071439951115469D"
            "0D2E2B80ECBCDD48DB88AC00000000")

        # block 80974
        block_80974_id = '0000000000089F7910F6755C10EA2795EC368A29B435D80770AD78493A6FECF1'.lower()
        block_80974_data = h2b(
            "010000007480150B299A16BBCE5CCDB1D1BBC65CFC5893B01E6619107C552000000000"
            "007900A2B203D24C69710AB6A94BEB937E1B1ADD64C2327E268D8C3E5F8B41DBED8796"
            "974CED66471B204C324703010000000100000000000000000000000000000000000000"
            "00000000000000000000000000FFFFFFFF0804ED66471B024001FFFFFFFF0100F2052A"
            "010000004341045FEE68BAB9915C4EDCA4C680420ED28BBC369ED84D48AC178E1F5F7E"
            "EAC455BBE270DABA06802145854B5E29F0A7F816E2DF906E0FE4F6D5B4C9B92940E4F0"
            "EDAC000000000100000001F7B30415D1A7BF6DB91CB2A272767C6799D721A4178AA328"
            "E0D77C199CB3B57F010000008A4730440220556F61B84F16E637836D2E74B8CB784DE4"
            "0C28FE3EF93CCB7406504EE9C7CAA5022043BD4749D4F3F7F831AC696748AD8D8E79AE"
            "B4A1C539E742AA3256910FC88E170141049A414D94345712893A828DE57B4C2054E2F5"
            "96CDCA9D0B4451BA1CA5F8847830B9BE6E196450E6ABB21C540EA31BE310271AA00A49"
            "ED0BA930743D1ED465BAD0FFFFFFFF0200E1F505000000001976A914529A63393D63E9"
            "80ACE6FA885C5A89E4F27AA08988ACC0ADA41A000000001976A9145D17976537F30886"
            "5ED533CCCFDD76558CA3C8F088AC00000000010000000165148D894D3922EF5FFDA962"
            "BE26016635C933D470C8B0AB7618E869E3F70E3C000000008B48304502207F5779EBF4"
            "834FEAEFF4D250898324EB5C0833B16D7AF4C1CB0F66F50FCF6E85022100B78A65377F"
            "D018281E77285EFC31E5B9BA7CB7E20E015CF6B7FA3E4A466DD195014104072AD79E0A"
            "A38C05FA33DD185F84C17F611E58A8658CE996D8B04395B99C7BE36529CAB7606900A0"
            "CD5A7AEBC6B233EA8E0FE60943054C63620E05E5B85F0426FFFFFFFF02404B4C000000"
            "00001976A914D4CAA8447532CA8EE4C80A1AE1D230A01E22BFDB88AC8013A0DE010000"
            "001976A9149661A79AE1F6D487AF3420C13E649D6DF3747FC288AC00000000")

        block_80971 = network.block.from_bin(block_80971_data)
        self.assertEqual(block_80971.id(), block_80971_id)
        block_80974 = network.block.from_bin(block_80974_data)
        self.assertEqual(block_80974.id(), block_80974_id)

        tx_db = {tx.hash(): tx for tx in block_80971.txs}

        tx_to_validate = block_80974.txs[2]
        self.assertEqual("OP_DUP OP_HASH160 [d4caa8447532ca8ee4c80a1ae1d230a01e22bfdb] OP_EQUALVERIFY OP_CHECKSIG",
                         network.script.disassemble(tx_to_validate.txs_out[0].script))
        self.assertEqual(tx_to_validate.id(), "7c4f5385050c18aa8df2ba50da566bbab68635999cc99b75124863da1594195b")

        tx_to_validate.unspents_from_db(tx_db)
        self.assertEqual(tx_to_validate.bad_solution_count(), 0)

        # now, let's corrupt the Tx and see what happens
        tx_out = tx_to_validate.txs_out[1]

        disassembly = network.script.disassemble(tx_out.script)
        tx_out.script = network.script.compile(disassembly)

        self.assertEqual(tx_to_validate.bad_solution_count(), 0)

        disassembly = disassembly.replace("9661a79ae1f6d487af3420c13e649d6df3747fc2",
                                          "9661a79ae1f6d487af3420c13e649d6df3747fc3")

        tx_out.script = network.script.compile(disassembly)

        self.assertEqual(tx_to_validate.bad_solution_count(), 1)
        self.assertFalse(tx_to_validate.is_solution_ok(0))

    def test_validate_two_inputs(self):
        def tx_from_b64(h):
            d = binascii.a2b_base64(h.encode("utf8"))
            return network.tx.from_bin(d)
        # tx_0 = c9989d984c97128b03b9f118481c631c584f7aa42b578dbea6194148701b053d
        # This is the one we're going to validate. It has inputs from
        #  tx_1 = b52201c2741d410b70688335afebba0d58f8675fa9b6c8c54becb0d7c0a75983
        # and tx_2 = 72151f65db1d8594df90778639a4c0c17c1e303af01de0d04af8fac13854bbfd
        TX_0_HEX = (
            "AQAAAAKDWafA17DsS8XItqlfZ/hYDbrrrzWDaHALQR10wgEitQAAAACLSDBFAiAnyvQ1P7"
            "b8+84JbBUbE1Xtgrd0KNpD4eyVTNU/burbtgIhAOS8T1TrhXkGXQTGbLSEJy5uvZMGEzOj"
            "ITxO+DrykiPlAUEE3yJcIB5OCpaDjrop+N3bm8h9PKw8bF/YB4v3yD+VeQf4fXdUZ9hJJS"
            "nFeJ+QeJrC7q3Y23QSYeYbW/AfA3D5G//////9u1Q4wfr4StDgHfA6MB58wcCkOYZ3kN+U"
            "hR3bZR8VcgAAAACLSDBFAiAN6ZQr+9HTgmF57EsPyXIhQ6J5M4lgwlj/tJTShZ+toQIhAL"
            "0U1i9yiCEm75uCEp8uRaySqS7P4x7A+L2Vr5kS+7ANAUEEkSqVI6gw1scM0GuJWgMh4jpW"
            "KJA0yOl03uQaV/jHURn+HswOIORzvsG9qQY1/9BZgDPaMuI5U5JlyA3WkhLxgf////8Ctk"
            "SUzxAAAAAZdqkULXTu3lp2t/wMSuvqbifOSj9/kvmIrAAoa+4AAAAAGXapFF3ySpVdjz9V"
            "8fRKvzDqXQRcmowSiKwAAAAA")
        TX_1_HEX = (
            "AQAAAAEL3YmFDcZpf4SH7uN1IBmMoBd4OhmTp4EAQ8A0ZQ3tiwAAAACKRzBEAiA4Fkl8lk"
            "JSeLtWHsp1j0h7y0KKFmqxhDR0CK0HnmZWBQIgDSTDenor3zbNqTs+FApeDl8DKCz1xGQC"
            "JQN0/sp00VABQQQzSNc33wdDXA/F9y9/hAR88q6Se6vRCHEC7dYgbIp1pgxqGzrWXQroGk"
            "QLhnAbn/fDhUoVbCgM/UHXYmjXlhdO/////wI3HGlfEQAAABl2qRRM+dhUVUjeAlb0jEsH"
            "JrFClGGSZ4isMAYVCgAAAAAZdqkUgnSLXoYTeOKFFRdtLYxWcGZ2Ht2IrAAAAAA=")
        TX_2_HEX = (
            "AQAAAAFDjBbw61AYUWMx+3moZ2vb9dvLKydOSFIwcfBTjG0QSgEAAACKRzBEAiA5WWKhR4"
            "8OI60ZDCXnOru/FH6NvuTGhRLggjbpJB2dhgIgKp0FFL0ClSCxxqGjYneDinvgROGSw6Dt"
            "Vtvflrhaom8BQQR50YjAg1e5qRkP4ER29ec5jKfzk3DHJhS7Si0sEbvNIJMfjjbZfZWtJi"
            "15wHZhuHh4e3G6SWMdJLHH5pgbseFh/////wLPE5deAAAAABl2qRSmRdbMvv5fEbgFD1Yk"
            "taBU9zQTW4iswJ7mBQAAAAAZdqkU4E5+Is4tr+8bPU6ELYHSvz/Ng0eIrAAAAAA=")
        tx_0 = tx_from_b64(TX_0_HEX)
        self.assertEqual(tx_0.id(), "c9989d984c97128b03b9f118481c631c584f7aa42b578dbea6194148701b053d")
        tx_1 = tx_from_b64(TX_1_HEX)
        self.assertEqual(tx_1.id(), "b52201c2741d410b70688335afebba0d58f8675fa9b6c8c54becb0d7c0a75983")
        tx_2 = tx_from_b64(TX_2_HEX)
        self.assertEqual(tx_2.id(), "72151f65db1d8594df90778639a4c0c17c1e303af01de0d04af8fac13854bbfd")

        TX_DB = {tx.hash(): tx for tx in [tx_0, tx_1, tx_2]}

        tx_to_validate = tx_0
        self.assertEqual("OP_DUP OP_HASH160 [2d74eede5a76b7fc0c4aebea6e27ce4a3f7f92f9] OP_EQUALVERIFY OP_CHECKSIG",
                         network.script.disassemble(tx_to_validate.txs_out[0].script))
        self.assertEqual(tx_to_validate.id(), "c9989d984c97128b03b9f118481c631c584f7aa42b578dbea6194148701b053d")

        tx_to_validate.unspents_from_db(TX_DB)
        self.assertEqual(tx_to_validate.bad_solution_count(), 0)

        # now let's mess with signatures
        disassembly = network.script.disassemble(tx_to_validate.txs_in[0].script)
        tx_to_validate.txs_in[0].script = network.script.compile(disassembly)
        self.assertEqual(tx_to_validate.bad_solution_count(), 0)
        disassembly = disassembly.replace("353fb6fcfbce09", "353fb6fcfbce19")
        tx_to_validate.txs_in[0].script = network.script.compile(disassembly)
        self.assertEqual(tx_to_validate.bad_solution_count(), 1)
        self.assertFalse(tx_to_validate.is_solution_ok(0))

        tx_to_validate = tx_from_b64(TX_0_HEX)
        tx_to_validate.unspents_from_db(TX_DB)
        self.assertEqual(tx_to_validate.bad_solution_count(), 0)
        disassembly = network.script.disassemble(tx_to_validate.txs_in[1].script)
        disassembly = disassembly.replace("960c258ffb494d2859f", "960d258ffb494d2859f")
        tx_to_validate.txs_in[1].script = network.script.compile(disassembly)
        self.assertEqual(tx_to_validate.bad_solution_count(), 1)
        self.assertFalse(tx_to_validate.is_solution_ok(1))

        # futz with signature on tx_1
        tx_to_validate = tx_from_b64(TX_0_HEX)
        original_tx_hash = tx_1.hash()
        disassembly = network.script.disassemble(tx_1.txs_out[0].script)
        disassembly = disassembly.replace("4cf9d8545548de0256f48c4b0726b14294619267",
                                          "4cf9d8545548de1256f48c4b0726b14294619267")
        tx_1.txs_out[0].script = network.script.compile(disassembly)
        TX_DB[original_tx_hash] = tx_1
        tx_to_validate.unspents_from_db(TX_DB, ignore_missing=True)
        self.assertEqual(tx_to_validate.bad_solution_count(), 1)
        self.assertFalse(tx_to_validate.is_solution_ok(0))

        # fix it up again
        TX_DB[original_tx_hash] = tx_from_b64(TX_1_HEX)
        tx_to_validate.unspents_from_db(TX_DB)
        self.assertEqual(tx_to_validate.bad_solution_count(), 0)

        # futz with signature on tx_2
        tx_to_validate = tx_from_b64(TX_0_HEX)
        original_tx_hash = tx_2.hash()
        disassembly = network.script.disassemble(tx_2.txs_out[0].script)
        disassembly = disassembly.replace("a645d6ccbefe5f11b8050f5624b5a054f734135b",
                                          "a665d6ccbefe5f11b8050f5624b5a054f734135b")
        tx_2.txs_out[0].script = network.script.compile(disassembly)
        TX_DB[original_tx_hash] = tx_2
        tx_to_validate.unspents_from_db(TX_DB, ignore_missing=True)
        self.assertEqual(tx_to_validate.bad_solution_count(), 1)
        self.assertFalse(tx_to_validate.is_solution_ok(1))

        # fix it up again
        TX_DB[original_tx_hash] = tx_from_b64(TX_2_HEX)
        tx_to_validate.unspents_from_db(TX_DB)
        self.assertEqual(tx_to_validate.bad_solution_count(), 0)

    def _make_tx(self, input_script, other_scripts=[]):
        cv = int(50*1e8)

        key = self._key
        sec = key.sec()
        wif = key.wif()
        address = key.address()
        p2sh_lookup = network.tx.solve.build_p2sh_lookup(other_scripts)

        coinbase_tx = network.tx.coinbase_tx(public_key_sec=sec, coin_value=cv)
        coinbase_tx.txs_out[0].script = input_script
        spendable = coinbase_tx.tx_outs_as_spendable()[0]
        payables = [(address, cv)]
        tx = network.tx_utils.create_signed_tx(
            spendables=[spendable], payables=payables, wifs=[wif], p2sh_lookup=p2sh_lookup)
        tx.unspents = [spendable]
        print(tx.as_hex(include_unspents=True))
        return tx

    def test_validate_p2pkh(self):
        us_1 = network.contract.for_p2pkh(self._key.hash160())
        tx = self._make_tx(us_1)
        tx.check_solution(0)

    def test_validate_p2s_of_p2pkh(self):
        us_1 = network.contract.for_p2pkh(self._key.hash160())
        us_2 = network.contract.for_p2s(us_1)
        tx = self._make_tx(us_2, [us_1])
        tx.check_solution(0)

    def test_validate_p2pkh_wit(self):
        us_1 = network.contract.for_p2pkh_wit(self._key.hash160())
        tx = self._make_tx(us_1)
        tx.check_solution(0)

    def test_validate_p2s_wit_of_p2pkh(self):
        us_1 = network.contract.for_p2pkh_wit(self._key.hash160())
        us_2 = network.contract.for_p2s(us_1)
        tx = self._make_tx(us_2, [us_1])
        self.assertEqual(tx.id(), "1e5d967a3778bfa4e0d90f35f59530e8033a36bd7fd1d9e617c504054b89bd3a")
        tx.check_solution(0)

    def test_validate_p2s_of_p2s_wit_of_p2pkh(self):
        us_1 = network.contract.for_p2pkh(self._key.hash160())
        us_2 = network.contract.for_p2s_wit(us_1)
        us_3 = network.contract.for_p2s(us_2)
        tx = self._make_tx(us_3, [us_1, us_2])
        self.assertEqual(tx.id(), "54a518b82b464744951531270c1bcec133c515fcdbe9d70c6141e067a62ff640")
        tx.check_solution(0)


if __name__ == "__main__":
    unittest.main()



import unittest

from pycoin.blockchain.BlockChain import BlockChain


class FakeBlock(object):
    def __init__(self, n, previous_block_hash=None):
        if previous_block_hash is None:
            previous_block_hash = n - 1
        self.n = n
        self.previous_block_hash = previous_block_hash
        self.difficulty = 1

    def hash(self):
        return self.n


def longest_block_chain(self):
    c = []
    for idx in range(self.length()):
        c.append(self.hash_for_index(idx))
    return c


def longest_locked_block_chain(self):
    c = []
    for idx in range(self.locked_length(), self.length()):
        c.append(self.hash_for_index(idx))
    return c


parent_for_0 = "motherless"


class BlockchainTestCase(unittest.TestCase):

    def test_basic(self):
        BC = BlockChain(parent_for_0)
        ITEMS = [FakeBlock(i) for i in range(100)]
        ITEMS[0] = FakeBlock(0, parent_for_0)

        assert longest_block_chain(BC) == []
        assert BC.length() == 0
        assert BC.locked_length() == 0
        assert set(BC.chain_finder.missing_parents()) == set()
        assert BC.parent_hash == parent_for_0
        assert BC.index_for_hash(0) is None
        assert BC.index_for_hash(-1) is None

        ops = BC.add_headers(ITEMS[:5])
        assert ops == [("add", ITEMS[i], i) for i in range(5)]
        assert BC.parent_hash == parent_for_0
        assert longest_block_chain(BC) == list(range(5))
        assert BC.length() == 5
        assert BC.locked_length() == 0
        assert set(BC.chain_finder.missing_parents()) == {parent_for_0}
        for i in range(5):
            v = BC.tuple_for_index(i)
            assert v[0] == i
            assert v[1] == parent_for_0 if i == 0 else i
        assert BC.index_for_hash(-1) is None

        ops = BC.add_headers(ITEMS[:7])
        assert ops == [("add", ITEMS[i], i) for i in range(5, 7)]
        assert BC.parent_hash == parent_for_0
        assert longest_block_chain(BC) == list(range(7))
        assert BC.length() == 7
        assert BC.locked_length() == 0
        assert set(BC.chain_finder.missing_parents()) == {parent_for_0}
        for i in range(7):
            v = BC.tuple_for_index(i)
            assert v[0] == i
            assert v[1] == parent_for_0 if i == 0 else i
        assert BC.index_for_hash(-1) is None

        ops = BC.add_headers(ITEMS[10:14])
        assert ops == []
        assert BC.parent_hash == parent_for_0
        assert longest_block_chain(BC) == [0, 1, 2, 3, 4, 5, 6]
        assert BC.locked_length() == 0
        assert BC.locked_length() == 0
        assert BC.length() == 7
        assert set(BC.chain_finder.missing_parents()) == {parent_for_0, 9}
        for i in range(7):
            v = BC.tuple_for_index(i)
            assert v[0] == i
            assert v[1] == parent_for_0 if i == 0 else i
        assert BC.index_for_hash(-1) is None

        ops = BC.add_headers(ITEMS[7:10])
        assert ops == [("add", ITEMS[i], i) for i in range(7, 14)]
        assert longest_block_chain(BC) == list(range(14))
        assert set(BC.chain_finder.missing_parents()) == {parent_for_0}
        assert BC.parent_hash == parent_for_0
        assert BC.locked_length() == 0
        assert BC.length() == 14
        for i in range(14):
            v = BC.tuple_for_index(i)
            assert v[0] == i
            assert v[1] == parent_for_0 if i == 0 else i
        assert BC.index_for_hash(-1) is None

        ops = BC.add_headers(ITEMS[90:])
        assert ops == []
        assert longest_block_chain(BC) == list(range(14))
        assert set(BC.chain_finder.missing_parents()) == {parent_for_0, 89}
        assert BC.parent_hash == parent_for_0
        assert BC.locked_length() == 0
        assert BC.length() == 14
        for i in range(14):
            v = BC.tuple_for_index(i)
            assert v[0] == i
            assert v[1] == parent_for_0 if i == 0 else i
        assert BC.index_for_hash(-1) is None

        ops = BC.add_headers(ITEMS[14:90])
        assert ops == [("add", ITEMS[i], i) for i in range(14, 100)]
        assert longest_block_chain(BC) == list(range(100))
        assert set(BC.chain_finder.missing_parents()) == {parent_for_0}
        assert BC.parent_hash == parent_for_0
        assert BC.locked_length() == 0
        assert BC.length() == 100
        for i in range(100):
            v = BC.tuple_for_index(i)
            assert v[0] == i
            assert v[1] == parent_for_0 if i == 0 else i
        assert BC.index_for_hash(-1) is None

    def test_fork(self):
        parent_for_0 = b'\0' * 32
        # 0 <= 1 <= ... <= 5 <= 6
        # 3 <= 301 <= 302 <= 303 <= 304 <= 305

        # parent_for_0 = "motherless"
        BC = BlockChain(parent_for_0)
        ITEMS = dict((i, FakeBlock(i)) for i in range(7))
        ITEMS[0] = FakeBlock(0, parent_for_0)

        ITEMS.update(dict((i, FakeBlock(i)) for i in range(301, 306)))
        ITEMS[301] = FakeBlock(301, 3)

        assert longest_block_chain(BC) == []
        assert BC.locked_length() == 0
        assert BC.length() == 0
        assert set(BC.chain_finder.missing_parents()) == set()

        # send them all except 302
        ops = BC.add_headers((ITEMS[i] for i in ITEMS.keys() if i != 302))
        assert ops == [("add", ITEMS[i], i) for i in range(7)]
        assert set(BC.chain_finder.missing_parents()) == set([parent_for_0, 302])

        # now send 302
        ops = BC.add_headers([ITEMS[302]])

        # we should see a change
        expected = [("remove", ITEMS[i], i) for i in range(6, 3, -1)]
        expected += [("add", ITEMS[i], i+4-301) for i in range(301, 306)]
        assert ops == expected
        assert set(BC.chain_finder.missing_parents()) == set([parent_for_0])

    def test_callback(self):
        R = []

        def the_callback(blockchain, ops):
            R.extend(ops)

        parent_for_0 = b'\0' * 32
        # same as test_fork, above

        BC = BlockChain(parent_for_0)
        BC.add_change_callback(the_callback)

        ITEMS = dict((i, FakeBlock(i)) for i in range(7))
        ITEMS[0] = FakeBlock(0, parent_for_0)

        ITEMS.update(dict((i, FakeBlock(i)) for i in range(301, 306)))
        ITEMS[301] = FakeBlock(301, 3)

        # send them all except 302
        BC.add_headers((ITEMS[i] for i in ITEMS.keys() if i != 302))

        # now send 302
        BC.add_headers([ITEMS[302]])

        expected = [("add", ITEMS[i], i) for i in range(7)]
        expected += [("remove", ITEMS[i], i) for i in range(6, 3, -1)]
        expected += [("add", ITEMS[i], i+4-301) for i in range(301, 306)]

        assert R == expected

    def test_large(self):
        SIZE = 3000
        ITEMS = [FakeBlock(i) for i in range(SIZE)]
        ITEMS[0] = FakeBlock(0, parent_for_0)
        BC = BlockChain(parent_for_0)
        assert longest_block_chain(BC) == []
        assert BC.locked_length() == 0
        assert BC.length() == 0
        assert set(BC.chain_finder.missing_parents()) == set()

        ops = BC.add_headers(ITEMS)
        assert ops == [("add", ITEMS[i], i) for i in range(SIZE)]
        assert longest_block_chain(BC) == list(range(SIZE))
        assert set(BC.chain_finder.missing_parents()) == {parent_for_0}
        assert BC.parent_hash == parent_for_0
        assert BC.locked_length() == 0
        assert BC.length() == SIZE
        for i in range(SIZE):
            v = BC.tuple_for_index(i)
            assert v[0] == i
            assert v[1] == parent_for_0 if i == 0 else i
        assert BC.index_for_hash(-1) is None

    def test_chain_locking(self):
        SIZE = 2000
        COUNT = 200
        ITEMS = [FakeBlock(i, i-1) for i in range(SIZE*COUNT)]
        ITEMS[0] = FakeBlock(0, parent_for_0)
        BC = BlockChain(parent_for_0)
        assert longest_block_chain(BC) == []
        assert BC.locked_length() == 0
        assert BC.length() == 0
        assert set(BC.chain_finder.missing_parents()) == set()

        for i in range(COUNT):
            start, end = i*SIZE, (i+1)*SIZE
            lock_start = max(0, start-10)
            expected_parent = lock_start-1 if lock_start else parent_for_0
            assert BC.length() == start
            assert BC.locked_length() == lock_start
            ops = BC.add_headers(ITEMS[start:end])
            assert ops == [("add", ITEMS[i], i) for i in range(start, end)]
            assert longest_locked_block_chain(BC) == list(range(lock_start, end))
            assert set(BC.chain_finder.missing_parents()) == {expected_parent}
            assert BC.parent_hash == expected_parent
            assert BC.locked_length() == lock_start
            assert BC.length() == end
            for i in range(start, end):
                v = BC.tuple_for_index(i)
                assert v[0] == i
                assert v[1] == parent_for_0 if i == 0 else i
            assert BC.index_for_hash(-1) is None
            assert BC.locked_length() == max(0, lock_start)
            BC.lock_to_index(end-10)
            assert BC.locked_length() == end-10



import io
import struct

from pycoin.encoding.hash import double_sha256
from pycoin.encoding.hexbytes import b2h_rev
from pycoin.satoshi.satoshi_int import parse_satoshi_int
from pycoin.satoshi.satoshi_streamer import Streamer, STREAMER_FUNCTIONS

from .InvItem import InvItem
from .PeerAddress import PeerAddress

# definitions of message structures and types
# L: 4 byte integer
# 6: 6 byte integer
# Q: 8 byte long integer
# S: unicode string encoded using utf-8
# [v]: array of InvItem objects
# [LA]: array of (L, PeerAddress) tuples
# b: boolean
# A: PeerAddress object
# B: Block object
# T: Tx object
# O: optional boolean


STANDARD_P2P_MESSAGES = {
    'version': (
        "version:L services:Q timestamp:Q remote_address:A local_address:A"
        " nonce:Q subversion:S last_block_index:L relay:O"
    ),
    'verack': "",
    'addr': "date_address_tuples:[LA]",
    'inv': "items:[v]",
    'getdata': "items:[v]",
    'notfound': "items:[v]",
    'reject': "message:S code:1 reason:S data:#",
    'getblocks': "version:L hashes:[#] hash_stop:#",
    'getheaders': "version:L hashes:[#] hash_stop:#",
    'sendheaders': "",
    'tx': "tx:T",
    'block': "block:B",
    'headers': "headers:[zI]",
    'getaddr': "",
    'mempool': "",
    'feefilter': "fee_filter_value:Q",
    'sendcmpct': "enabled:b version:Q",
    'cmpctblock': "header_hash:# nonce:Q short_ids:[6] prefilled_txs:[IT]",
    'getblocktxn': "header_hash:# indices:[I]",
    'blocktxn': "header_hash:# txs:[T]",
    # 'checkorder': obsolete
    # 'submitorder': obsolete
    # 'reply': obsolete
    'ping': "nonce:Q",
    'pong': "nonce:Q",
    'filterload': "filter:[1] hash_function_count:L tweak:L flags:b",
    'filteradd': "data:[1]",
    'filterclear': "",
    'merkleblock': (
        "header:z total_transactions:L hashes:[#] flags:[1]"
    ),
    'alert': "payload:S signature:S",
}


def standard_messages():
    return dict(STANDARD_P2P_MESSAGES)


def _recurse(level_widths, level_index, node_index, hashes, flags, flag_index, tx_acc):
    idx, r = divmod(flag_index, 8)
    mask = (1 << r)
    flag_index += 1
    if flags[idx] & mask == 0:
        h = hashes.pop()
        return h, flag_index

    if level_index == len(level_widths) - 1:
        h = hashes.pop()
        tx_acc.append(h)
        return h, flag_index

    # traverse the left
    left_hash, flag_index = _recurse(
        level_widths, level_index+1, node_index*2, hashes, flags, flag_index, tx_acc)

    # is there a right?
    if node_index*2+1 < level_widths[level_index+1]:
        right_hash, flag_index = _recurse(
            level_widths, level_index+1, node_index*2+1, hashes, flags, flag_index, tx_acc)

        if left_hash == right_hash:
            raise ValueError("merkle hash has same left and right value at node %d" % node_index)
    else:
        right_hash = left_hash

    return double_sha256(left_hash + right_hash), flag_index


def post_unpack_merkleblock(d, f):
    """
    A post-processing "post_unpack" to merkleblock messages.

    It validates the merkle proofs (throwing an exception if there's
    an error), and returns the list of transaction hashes in "tx_hashes".

    The transactions are supposed to be sent immediately after the merkleblock message.
    """
    level_widths = []
    count = d["total_transactions"]
    while count > 1:
        level_widths.append(count)
        count += 1
        count //= 2
    level_widths.append(1)
    level_widths.reverse()

    tx_acc = []
    flags = d["flags"]
    hashes = list(reversed(d["hashes"]))
    left_hash, flag_index = _recurse(level_widths, 0, 0, hashes, flags, 0, tx_acc)

    if len(hashes) > 0:
        raise ValueError("extra hashes: %s" % hashes)

    idx, r = divmod(flag_index-1, 8)
    if idx != len(flags) - 1:
        raise ValueError("not enough flags consumed")

    if flags[idx] > (1 << (r+1))-1:
        raise ValueError("unconsumed 1 flag bits set")

    if left_hash != d["header"].merkle_root:
        raise ValueError(
            "merkle root %s does not match calculated hash %s" % (
                b2h_rev(d["header"].merkle_root), b2h_rev(left_hash)))

    d["tx_hashes"] = tx_acc
    return d


def _make_parser(streamer, the_struct):
    "Return a function that parses the given structure into a dict"
    struct_items = [s.split(":") for s in the_struct.split()]
    names = [s[0] for s in struct_items]
    types = ''.join(s[1] for s in struct_items)

    def f(message_stream):
        return streamer.parse_as_dict(names, types, message_stream)
    return f


def make_post_unpack_alert(streamer):
    """
    Post-processor to "alert" message, to add an "alert_info" dictionary of parsed
    alert information.
    """
    the_struct = ("version:L relayUntil:Q expiration:Q id:L cancel:L setCancel:[L] minVer:L "
                  "maxVer:L setSubVer:[S] priority:L comment:S statusBar:S reserved:S")

    alert_submessage_parser = _make_parser(streamer, the_struct)

    def post_unpack_alert(d, f):
        d1 = alert_submessage_parser(io.BytesIO(d["payload"]))
        d["alert_info"] = d1
        return d
    return post_unpack_alert


def standard_parsing_functions(Block, Tx):
    """
    Return the standard parsing functions for a given Block and Tx class.
    The return value is expected to be used with the standard_streamer function.
    """
    def stream_block(f, block):
        assert isinstance(block, Block)
        block.stream(f)

    def stream_blockheader(f, blockheader):
        assert isinstance(blockheader, Block)
        blockheader.stream_header(f)

    def stream_tx(f, tx):
        assert isinstance(tx, Tx)
        tx.stream(f)

    def parse_int_6(f):
        b = f.read(6) + b'\0\0'
        return struct.unpack(b, "<L")[0]

    def stream_int_6(f, v):
        f.write(struct.pack(v, "<L")[:6])

    more_parsing = [
        ("A", (PeerAddress.parse, lambda f, peer_addr: peer_addr.stream(f))),
        ("v", (InvItem.parse, lambda f, inv_item: inv_item.stream(f))),
        ("T", (Tx.parse, stream_tx)),
        ("B", (Block.parse, stream_block)),
        ("z", (Block.parse_as_header, stream_blockheader)),
        ("1", (lambda f: struct.unpack("B", f.read(1))[0], lambda f, v: f.write(struct.pack("B", v)))),
        ("6", (parse_int_6, stream_int_6)),
        ("O", (lambda f: True if f.read(1) else False,
               lambda f, v: f.write(b'' if v is None else struct.pack("B", v)))),
    ]
    all_items = list(STREAMER_FUNCTIONS.items())
    all_items.extend(more_parsing)
    return all_items


def standard_streamer(parsing_functions, parse_satoshi_int=parse_satoshi_int):
    """
    Create a satoshi_streamer, which parses and packs using the bitcoin protocol
    (mostly the custom way arrays and integers are parsed and packed).
    """
    streamer = Streamer()
    streamer.register_array_count_parse(parse_satoshi_int)
    streamer.register_functions(parsing_functions)
    return streamer


def standard_message_post_unpacks(streamer):
    """
    The standard message post-processors: one for the version message,
    one for the alert message, and one for the merkleblock message.
    """
    return dict(alert=make_post_unpack_alert(streamer), merkleblock=post_unpack_merkleblock)


def make_parser_and_packer(streamer, message_dict, message_post_unpacks):
    """
    Create a parser and a packer for a peer's network messages.

    streamer:
        used in conjunction with the message_dict. The message_dict turns a message into
        a string specifying the fields, and this dictionary specifies how to pack or unpack
        fields to or from bytes
    message_dict:
        a dictionary specifying how to pack or unpack the various messages like "version"
    message_post_unpacks:
        a dictionary specifying functions to call to postprocess message to, for example
        extract submessages, like in "alert"
    """
    message_parsers = dict((k, _make_parser(streamer, v)) for k, v in message_dict.items())

    def parse_from_data(message_name, data):
        message_stream = io.BytesIO(data)
        parser = message_parsers.get(message_name)
        if parser is None:
            raise KeyError("unknown message: %s" % message_name)
        d = parser(message_stream)
        post_unpack = message_post_unpacks.get(message_name)
        if post_unpack:
            d = post_unpack(d, message_stream)
        return d

    def pack_from_data(message_name, **kwargs):
        the_struct = message_dict[message_name]
        if not the_struct:
            return b''
        f = io.BytesIO()
        the_fields = the_struct.split(" ")
        pairs = [t.split(":") for t in the_fields]
        for name, type in pairs:
            if type[0] == '[':
                streamer.stream_struct("I", f, len(kwargs[name]))
                for v in kwargs[name]:
                    if not isinstance(v, (tuple, list)):
                        v = [v]
                    streamer.stream_struct(type[1:-1], f, *v)
            else:
                streamer.stream_struct(type, f, kwargs[name])
        return f.getvalue()

    return parse_from_data, pack_from_data
